// DaysEaseView.cpp : implementation of the CDaysEaseView class
//
/*
	DaysEase - PC Magazine file download utility
	Copyright  1998-1999 Ziff-Davis, Inc.  All rights reserved.
	First published in PC Magazine, US Edition
	December 1, 1998.
	Written by Richard A. Ellingson
*/
#include "stdafx.h"
#include "DaysEase.h"

#include "DaysEaseDoc.h"
#include "DaysEaseView.h"
#include "MainFrm.h"
#include "winspool.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
extern CDaysEaseApp theApp;
/////////////////////////////////////////////////////////////////////////////
// CDaysEaseView

IMPLEMENT_DYNCREATE(CDaysEaseView, CView)

BEGIN_MESSAGE_MAP(CDaysEaseView, CView)
	//{{AFX_MSG_MAP(CDaysEaseView)
	ON_WM_SIZE()
	ON_COMMAND(IDC_SETUP, OnSetup)
	ON_WM_SETCURSOR()
	ON_WM_TIMER()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_LBUTTONDOWN()
	ON_WM_KEYDOWN()
	ON_WM_KILLFOCUS()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_DESTROY()
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	//}}AFX_MSG_MAP
	// Standard printing commands
	//ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT, OnFilePrint)
	//ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDaysEaseView construction/destruction

CDaysEaseView::CDaysEaseView()
{
	// We only have one View, so save the pointer.
	theApp.Set_View_Pointer(this);
	// Initialize some variables.
	fNote = 0;
	fEdit = 0;
	m_pPrintDialog = NULL;
	fEvents++;
	fCalPos = 0;
	fGrafPos = 0;
	fGontop = 0;
}

CDaysEaseView::~CDaysEaseView()
{
	// Get rid of the Print Dialog object to avoid memory leaks.
	delete m_pPrintDialog;
}

BOOL CDaysEaseView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CDaysEaseView drawing

void CDaysEaseView::OnDraw(CDC* pDC)
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	CMainFrame* pMain = theApp.Main_Pointer();
	// Null the modified flag after opening document from MRU list.
	if(fNewDoc)
		pDoc->SetModifiedFlag(FALSE);
	col = 0;
	row = 1;
	int k = 1;
	int nDays = 0;
	CTimeSpan cts;
	CTime ctCurrent_Date, ctFirst_Day, ctLast_Day;
	char chYear[5];
	CPen* pOldPen;

	// Put the current printer name in the second status bar pane.
	Set_Status_Text(1, theApp.PrinterName);
	// If the calendar should be transparent, set the backgound and drawing brush.
	if(pDoc->fTransparent)
	{
		pDC->SetBkMode(TRANSPARENT);
		pDC->SelectStockObject(NULL_BRUSH);
	}
	//If the month hasen't been set, then use todays date
	if(pDoc->nMonth < 1 || pDoc->nMonth > 12)
	{
		// Get todays date - must prefix with CTime::
		ctCurrent_Date = CTime::GetCurrentTime();
		// Get the year integer
		pDoc->nYear = ctCurrent_Date.GetYear();
		// Get the month integer
		pDoc->nMonth = ctCurrent_Date.GetMonth();
		// Get the day integer
		nDay = ctCurrent_Date.GetDay();
	}
	// Set the 'Month' combo box  - (0 referenced)
	pMain->m_CalendarDlg.m_Month.SetCurSel(pDoc->nMonth - 1);
	// Convert the year to a character string
	itoa(pDoc->nYear, chYear, 10);
	// Set the 'Year' combo box string
	pMain->m_CalendarDlg.m_Year.SelectString(-1, (LPCTSTR) chYear);
	// If document has changed, set the title.
	if(pDoc->fNewDoc)
		Set_Title();
	// If events need updating, get new Easter, in case the year is different.
	if(fEvents)
		Calculate_Easter();
	// Get a CTime object for the first day of this month
	ctFirst_Day = CTime(pDoc->nYear, pDoc->nMonth, 1, 0, 0, 0, 0);
	// Get a CTime object for the first day of next month
	// If this is December, increment year and set next month to 1
	if(pDoc->nMonth == 12)
		ctLast_Day = CTime(pDoc->nYear + 1, 1, 1, 0, 0, 0, 0);
	// Otherwise increment month only
	else
		ctLast_Day = CTime(pDoc->nYear, pDoc->nMonth + 1, 1, 0, 0, 0, 0);
	// Get a CTimeSpan value for the month
	// this method frees us from calculating leap years
	// because CTimeSpan does it automatically
	cts = ctLast_Day - ctFirst_Day;
	// Get the number of days for this month
	nDays = cts.GetDays();
	// If not printing, use a lighter pen so the calendar grid doesn't look too harsh.
	CPen pen(PS_SOLID, 1, RGB(127, 127, 127));
	if(!pDC->IsPrinting())
		pOldPen = pDC->SelectObject (&pen);
	// Initialize the calendar grid
	if(!pDC->IsPrinting())
	{// Not printing
		if(pDoc->fMonYr)
		{// Year mode.
			View_Year(pDC);
			// If we have an image selected for positioning or we've set the
			// fGontop flag by clicking the Imape tabbed page, then draw the image
			// on top of the calendar.
			if(fGrafPos || fGontop)
				Draw_Graphic(pDC);
			// We don't need to do anything else, so return.
			return;
		}
		else // Month mode.
			View_Month(pDC);
	}
	else
	{// Printing or print preview.
		int print_left, print_top, print_right, print_bottom;
		int left;
		int top;

		// Correct the left and top page coordinates if the calculation returns less than zero.
		if((left = physwidth - cxPage - 2*physoffsetx) < 0)
			left = 0;
		if((top = physheight - cyPage - 2*physoffsety) < 0)
			top = 0;
		// If we have an image, size it for the printed page.
		if(pDoc->fGraphic)
		{
			CRect rect;

			// When printing, we must remove our calculated margins.
			print_left = (int)(pDoc->gratio_left*physwidth-physoffsetx);
			print_top = (int)(pDoc->gratio_top*physheight-physoffsety);
			print_right = (int)(pDoc->gratio_right*physwidth-physoffsetx);
			print_bottom = (int)(pDoc->gratio_bottom*physheight-physoffsety);
			// Correct our page if any calculations resulted in values out of bounds.
			if(print_left < left)
				print_left = left;
			if(print_top < top)
				print_top = top;
			if(print_right > cxPage)
				print_right = cxPage;
			if(print_bottom > cyPage)
				print_bottom = cyPage;
			// Set the image rectangle.
			rect.SetRect(print_left, print_top, print_right, print_bottom);
	
			// Call the CDib class to draw the image with the correct color palette.
			CDib& dib = pDoc->m_dib;
			dib.UsePalette(pDC);
			dib.Draw(pDC, rect.TopLeft(), rect.Size());
		}
		// Prepare to draw the calendar.
		// When printing, we must remove our calculated margins.
		print_left = (int)(pDoc->ratio_left*physwidth-physoffsetx);
		print_top = (int)(pDoc->ratio_top*physheight-physoffsety);
		print_right = (int)(pDoc->ratio_right*physwidth-physoffsetx);
		print_bottom = (int)(pDoc->ratio_bottom*physheight-physoffsety);
		// Correct our page if any calculations resulted in values out of bounds.
		if(print_left < left)
			print_left = left;
		if(print_top < top)
			print_top = top;
		if(print_right > cxPage)
			print_right = cxPage;
		if(print_bottom > cyPage)
			print_bottom = cyPage;
		if(pDoc->fMonYr)
		{// Year mode.
			Print_Year(pDC, print_left, print_top, print_right, print_bottom);
			// We don't need to do anything else, so return.
			return;
		}
		else // Month mode.
			Print_Month(pDC, print_left, print_top, print_right, print_bottom);
	}
	// The remainder of this function is only performed in Month mode.
	row = 0;
	// Draw the first week starting with the first day
	for(col = 0; col < 7; col++)
	{
		pDC->Rectangle(m_rect[col][row]);
		// Save coordinates if this is the currently selected day
		if(k == nDay)
		{
			nCurX = col;
			nCurY = row;
		}
		// Only insert day numbers when we reach the first day of the month.
		if(col >= ctFirst_Day.GetDayOfWeek() -1)
		{
			Insert_Day_Number(pDC, &m_rect[col][row], k, nDays);
			k++;
		}
	}
	// Create the remaining days until the end of the month.
	for (row = 1; row < 6; row++)
	{
		for (col = 0; col < 7; col++)
		{
			// Make a correction so the right vertical line prints properly on the last day.
			if(k == nDays && row == 5 && pDC->IsPrinting())
				m_rect[col][row].right -= 1;
			pDC->Rectangle(m_rect[col][row]);
			// Save coordinates if this is the currently selected day
			if(k == nDay)
			{
				nCurX = col;
				nCurY = row;
			}
			// Only insert day numbers until we reach the last day of the month.
			if(k <= nDays)
			{
				Insert_Day_Number(pDC, &m_rect[col][row], k, nDays);
				k++;
			}
		}
	}
	// Insert previous and next mini months
	int premonth, nexmonth, preyear, nexyear;

	switch(pDoc->nMonth)
	{
	case 1:		// January.
		premonth = 12;
		nexmonth = 2;
		preyear = pDoc->nYear -1;
		nexyear = pDoc->nYear;
		break;
	case 12:	// December.
		premonth = 11;
		nexmonth = 1;
		preyear = pDoc->nYear;
		nexyear = pDoc->nYear +1;
		break;
	default:	// All other months.
		premonth = pDoc->nMonth -1;
		nexmonth = pDoc->nMonth +1;
		preyear = nexyear = pDoc->nYear;
	}
	// If there's room at the beginning of the month, put the mini months there.
	if(ctFirst_Day.GetDayOfWeek() > 2)
		col = row = 0;
	// Otherwise put them at the end.
	else
	{
		col = 5;
		row = 4;
	}
	// Draw the previous month.
	Mini_Month(pDC, m_rect[col][row], premonth,
		preyear, m_rect[col][row].Height()/7);
	// Draw the next month.
	Mini_Month(pDC, m_rect[col +1][row], nexmonth,
		nexyear, m_rect[col +1][row].Height()/7);
	// If not printing, restore the original pen.
	if(!pDC->IsPrinting())
		pDC->SelectObject(pOldPen);
	// If we have an image selected for positioning or we've set the
	// fGontop flag by clicking the Imape tabbed page, then draw the image
	// on top of the calendar.
	if((fGrafPos || fGontop) && !pDC->IsPrinting())
		Draw_Graphic(pDC);
	// If the Day Notes edit box needs updating, do it now.
	if(fEdit)
	{
		if(theApp.nDt_Swap == DT_RIGHT)	// Day number goes to right.
			pMain->m_CalendarDlg.m_Data.SetWindowText((char)0x09 + pDoc->GetSquare(nCurX, nCurY));
		else	// Day number goes to left.
			pMain->m_CalendarDlg.m_Data.SetWindowText(pDoc->GetSquare(nCurX, nCurY));
		// Clear our Day Notes edit flag.
		fEdit = 0;
	}
	// Clear our remaining View flags.
	fNote_Warn = 0;
	fEvents = 0;
	fNewDoc = 0;
}

/////////////////////////////////////////////////////////////////////////////
// CDaysEaseView printing

BOOL CDaysEaseView::OnPreparePrinting(CPrintInfo* pInfo)
{
// Print and Print Preview will have loaded the default printer settings, so
// we need to reload the current printer settings, even if they are the same.
// Note that hDC will get deleted automatically.
	HDC hDC = CreateDC(NULL, theApp.PrinterName, NULL, NULL);
	ResetDC(hDC, m_pPrintDialog->GetDevMode());
// Tell the print info structure to use our hDC
	pInfo->m_pPD->m_pd.hDC = hDC;
	// Our document only has one page, so set pages to 1.
	pInfo->m_nNumPreviewPages = 1;
	pInfo->SetMaxPage(1);
	return TRUE;
}

void CDaysEaseView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// Hide the DialogBar so the tabbed pages are gone.
	theApp.Main_Pointer()->m_MainDlgBar.ShowWindow(SW_HIDE);
}

void CDaysEaseView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
// Unhide the DialogBar
	theApp.Main_Pointer()->m_MainDlgBar.ShowWindow(SW_SHOW);
	Invalidate();
}

/////////////////////////////////////////////////////////////////////////////
// CDaysEaseView diagnostics

#ifdef _DEBUG
void CDaysEaseView::AssertValid() const
{
	CView::AssertValid();
}

void CDaysEaseView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
// AppWizard generated code, which we won't use.
/*
CDaysEaseDoc* CDaysEaseView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDaysEaseDoc)));
	return (CDaysEaseDoc*)m_pDocument;
}
*/
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDaysEaseView message handlers

void CDaysEaseView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	// Make sure we are visible, since OnSize is called at least once before we are.
	if(IsWindowVisible())
	{
		CRect rect;

		GetClientRect(rect);
		// Set the first pane to the client width.
		theApp.Main_Pointer()->m_wndStatusBar.SetPaneInfo(0,
				ID_SEPARATOR,  SBPS_NORMAL, rect.right -2);
		// Set the second pane arbitrarily long.
		theApp.Main_Pointer()->m_wndStatusBar.SetPaneInfo(1,
				ID_SEPARATOR,  SBPS_NORMAL, 1000);
	}
}

void CDaysEaseView::OnInitialUpdate()
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	CMainFrame* pMain = theApp.Main_Pointer();
	CView::OnInitialUpdate();

	// Initialize some variables.
	nDay = 1;
	fEdit++;
	fNewDoc++;
	fNote_Warn = 0;
	// Since these are never less than zero, we'll know they need to be set later.
	nCurX = -1;
	nCurY = -1;
	pDoc->SetModifiedFlag(FALSE);
	// Transfer the document fonts to our common CStrings.
	strcpy(pDoc->lfWeekday.lfFaceName, theApp.csWkdayFont);
	strcpy(pDoc->lfNotes.lfFaceName, theApp.csNotesFont);
	strcpy(pDoc->lfTitle.lfFaceName, theApp.csTitleFont);
	strcpy(pDoc->lfDate.lfFaceName, theApp.csDateFont);
	// Initialize some controls.
	pMain->m_StyleDlg.m_NoteLarge.SetCheck(theApp.nNoteLarge);
	pMain->m_StyleDlg.m_DateLarge.SetCheck(theApp.nDateLarge);
	pMain->m_StyleDlg.m_TitleLarge.SetCheck(pDoc->nTitleLarge);
	pMain->m_ImageDlg.m_Transparent.SetCheck(pDoc->fTransparent);
	pMain->m_StyleDlg.m_Title.SetWindowText(pDoc->cstitle);
	if(theApp.nDt_Swap == DT_RIGHT)
		pMain->m_CalendarDlg.m_wkdSwap.SetWindowText("Left");
	else
		pMain->m_CalendarDlg.m_wkdSwap.SetWindowText("Right");
	// Find out if the document has a preferred printer and try to find it.
	Create_Print_Dlg();
	// Put the printer name in the second statusbar pane.
	Set_Status_Text(1, theApp.PrinterName);
	// If the printer or its setup is different then we need to get new printer stats.
	if(Save_Printer_Changes(FALSE))
	{
		// Rest the print dialog DEVMODE structure.
		ResetDC(m_pPrintDialog->m_pd.hDC, m_pPrintDialog->GetDevMode());
		// Get the new stats.
		Get_Printer_Stats(m_pPrintDialog->m_pd.hDC);
	}
	// We had to get the printer stats before we could set the orientation button.
	if(theApp.Orientation == DMORIENT_LANDSCAPE)
		pMain->m_CalendarDlg.m_Orientation.SetWindowText("Portrait");
	else
		pMain->m_CalendarDlg.m_Orientation.SetWindowText("Landscape");
	// Setup the Image page, depending on whether we have a graphic in the document.
	pMain->m_ImageDlg.Enable_Controls(pDoc->fGraphic);
	// Save the document name in DaysEase.ini.
	CString DocName = pDoc->GetPathName();
	if(pDoc->fMonYr)
	{// It must be a Year document.
		if(!DocName.IsEmpty())
			theApp.WriteProfileString("DaysEase", "Year Doc", DocName);
		// Show the Calendar page in Year mode.
		pMain->m_CalendarDlg.Display_Year_Page();
	}
	else
	{// It must be a Month document.
		if(!DocName.IsEmpty())
			theApp.WriteProfileString("DaysEase", "Month Doc", DocName);
		// Show the Calendar page in Month mode.
		pMain->m_CalendarDlg.Display_Month_Page();
	}
}

void CDaysEaseView::Get_Printer_Stats(HDC hDC)
{
	// These are the width and height of the current page, e.g. 8 1/2 by 11,
	// expressed in printer device units.
	physwidth = ::GetDeviceCaps(hDC, PHYSICALWIDTH);
	physheight = ::GetDeviceCaps(hDC, PHYSICALHEIGHT);
	// These are width and height of the printable area, not including margins.
	// We subtract 10 because some printer drivers are slightly inaccurate.
	cxPage = ::GetDeviceCaps(hDC, HORZRES) - 10;
	cyPage = ::GetDeviceCaps(hDC, VERTRES) - 10;
	// These are the left and top margins.
	physoffsetx = ::GetDeviceCaps(hDC, PHYSICALOFFSETX);
	physoffsety = ::GetDeviceCaps(hDC, PHYSICALOFFSETY);
	// Save these variables.
	theApp.PrinterName = m_pPrintDialog->GetDevMode()->dmDeviceName;
	theApp.Orientation = m_pPrintDialog->GetDevMode()->dmOrientation;
	theApp.PaperSize = m_pPrintDialog->GetDevMode()->dmPaperSize;
	theApp.PrintColor = m_pPrintDialog->GetDevMode()->dmColor;
	// Save the printer name in DaysEase.ini.
	theApp.WriteProfileString("DaysEase", "Printer", theApp.PrinterName);

}

void CDaysEaseView::Set_Status_Text(int pane, CString text)
{
	// Copy whatever text is passed to the statusbar pane given.
	theApp.Main_Pointer()->m_wndStatusBar.SetPaneText(pane, text);
}

void CDaysEaseView::Set_Title()
{
	CString text, DocName("");

	if(!theApp.Doc_Pointer()->fMonYr)
	{// It must be a Month so get the name of the month and abbreviate it.
		theApp.Main_Pointer()->m_CalendarDlg.m_Month.GetWindowText(text);
		DocName = text.Left(3);
	}
	// Get the year.
	theApp.Main_Pointer()->m_CalendarDlg.m_Year.GetWindowText(text);
	// Add the year to the abbreviation, if any.
	DocName += text;
	// Tack on our unsaved identifier.
	// Note that when we save the document everything starting with the space before [unsaved]
	// will be truncated from the filename.  Curiously, this also applies to the characters, #%;/\
	// but not to the illegal filename characters, :*?"<>|
	// If you include one of those, then the file will never be saved
	// and there's no message indicating that it wasn't.  This is done in the MFC source file Doccore.cpp
	// within the function CDocument::DoSave().  IMHO, this needs to be updated, but then again, if they
	// update it, this source code will compile with a bug.  Maybe that's why they haven't update it.
	DocName += " [unsaved]";
	// Set the main window title.
	theApp.Doc_Pointer()->SetTitle(DocName);
	// Reset the new document flag.
	theApp.Doc_Pointer()->fNewDoc = 0;
}

void CDaysEaseView::Calculate_Easter()
{
	CMainFrame* pMain = theApp.Main_Pointer();
	CString day;
	// Carter's method
	//1.Calculate B = 225 - 11(Y MOD 19). 
	//2.Calculate D = ((B-21) MOD 30) + 21. 
	//3.If D is greater than 48 subtract 1 from it. 
	//4.Calculate E = (Y + [Y/4] + D + 1) MOD 7 . 
	//5.Calculate Q= D + 7 - E. 
	//6.If Q is less than 32 then Easter is in March (on the Qth day).
	//If Q is greater than 31 then Q - 31 is its date in April. 

	int Y = pMain->m_CalendarDlg.m_Year.GetCurSel();
	Y += 1970;
	int B = 225 - (11 * (Y % 19));
	int D = ((B-21) % 30) + 21;
	if(D > 48)
		D--;
	int E = (Y + (Y/4) + D + 1) % 7;
	int Q= D + 7 - E;
	char ch[3];
	if(Q < 32)
	{
		itoa(Q, ch, 10);
		csEaster = "Mar ";
	}
	else
	{
		itoa(Q - 31, ch, 10);
		csEaster = "Apr ";
	}
	day = ch;
	// The following two lines were added in version 1.1 to fix a bug where Easter dates falling before
	// the tenth of the month were not recognized.  Found by Larry Howson.
	if(day.GetLength() == 1)
		day = "0" + day;
	// Add the day and a tab character.
	csEaster += day + (char)0x09;
}

void CDaysEaseView::View_Year(CDC * pDC)
{// Called by OnDraw()
	CMainFrame* pMain = theApp.Main_Pointer();
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	int ntheight; // title box height
	int nTop, nLeft, nRight, nBottom;
	int nTW, nTW7, nTH, nTH6;
	int L_margin, T_margin, R_margin, B_margin;
	char ch[5];
	itoa(pDoc->nYear, ch,10);
	CString csYear = ch;
	CRect rect;
	CGdiObject* pOldPen = NULL;	

	// Setup the page.
	Draw_Calendar(pDC);
	// If the Image needs to be behind the calendar, draw it now.
	if(!fGrafPos && !fGontop)
		Draw_Graphic(pDC);
	// Make the tracker rectangle
	// Left margin = left * client width.
	L_margin = (int)(pDoc->ratio_left*dCW);
	// Top margin = top * client height.
	T_margin = (int)(pDoc->ratio_top*dCH);
	// Total width = (right - left) * client width.
	nTW = (int)(pDoc->ratio_right*dCW-pDoc->ratio_left*dCW);
	// Right margin = Total width + left margin.
	R_margin = nTW+L_margin;
	// Calculate the width of the columns.
	nTW7 = nTW/pMain->m_CalendarDlg.nYrx;
	// Calculate a title box height.
	int min = (int)((dCH*(pDoc->ratio_bottom-pDoc->ratio_top))/20);
	int max = (int)((nTW)/20);
	if(min < max)
		ntheight = min;
	else
		ntheight = max;
	// If either the date or the title should be large, multiple by 3.
	if(theApp.nDateLarge || pDoc->nTitleLarge)
		ntheight = ntheight*3;
	else // Otherwise multiple by 1.5.
		ntheight = (int)(ntheight*1.5);
	// Total height = ((bottom - top) * client height) - the title box height.
	nTH = (int)((dCH*pDoc->ratio_bottom)-(dCH*pDoc->ratio_top)-ntheight);
	// Bottom margin = total height - top margin.
	B_margin = nTH+T_margin;
	// Calculate the width of the rows.
	nTH6 = nTH/pMain->m_CalendarDlg.nYry;
	// Set a rectangle, adding back the title box height.
	rect.SetRect(L_margin, T_margin, R_margin, B_margin+ntheight);
	// Draw the rectangle.
	pDC->Rectangle(rect);
	// Copy our calculated rectangle into the calendar tracker rectangle to avoid discrepancies.
	m_calendar.m_rect.CopyRect(rect);
	// Show the tracker rectangle border if we are positioning
	if(fCalPos)
		m_calendar.Draw(pDC);
	// Make the title rectangle
	rect.SetRect(L_margin, T_margin, R_margin, T_margin+ntheight);
	// Draw the rectangle.
	pDC->Rectangle(rect);
	// Make a slightly smaller rectangle for the text.
	rect.DeflateRect((int)(rect.Width()*.01), (int)(rect.Height()*.08));
	// Draw the title and the date.
	Draw_Title_Date(pDC, rect, FALSE);
	// Make the calendar grid
	int month = 0;
	for(row = 0; row < pMain->m_CalendarDlg.nYry; row++)
	{
		for(col = 0; col < pMain->m_CalendarDlg.nYrx; col++)
		{// Calculate the month boxes.
			month++;
			nLeft = (col * nTW7)+L_margin;
			nTop = (row * nTH6)+T_margin+ntheight;
			nRight = nLeft + nTW7+1;
			if(col == pMain->m_CalendarDlg.nYrx-1) nRight = nTW+L_margin;
			nBottom = nTop + nTH6+1;
			if(row == pMain->m_CalendarDlg.nYry-1) nBottom = nTH+T_margin+ntheight;
			rect.SetRect(nLeft, nTop, nRight, nBottom);
			// Draw the month.
			Mini_Month(pDC, rect, month, pDoc->nYear, rect.Height()/7);
		}
	}
}

void CDaysEaseView::Draw_Graphic(CDC * pDC)
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	
	// If we don't have an Image, bail out.
	if(!pDoc->fGraphic)
		return;
	// Make the graphic tracker rectangle
	m_graphic.m_rect.SetRect((int)((pDoc->gratio_left*dCW)), (int)((pDoc->gratio_top*dCH)),
		(int)((pDoc->gratio_right*dCW)), (int)((pDoc->gratio_bottom*dCH)));
	// If we are positioning, draw the tracker border.
	if(fGrafPos)
	{
		pDC->LPtoDP(m_graphic.m_rect);// tracker wants device coords
		m_graphic.Draw(pDC);
	}
	// Set up the palette.
	pDoc->m_dib.SetSystemPalette(pDC);
	pDoc->m_dib.UsePalette(pDC);
	// Draw the Image in the tracker rectangle.
	pDoc->m_dib.Draw(pDC, m_graphic.m_rect.TopLeft(), m_graphic.m_rect.Size());
}

void CDaysEaseView::View_Month(CDC * pDC)
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	CFont Weekday;
	int nwdheight; // weekday box height
	int ntheight; // title box height
	int nTop, nLeft, nRight, nBottom;
	int nTW, nTW7, nTH, nTH6;
	int L_margin, T_margin, R_margin, B_margin;
	COLORREF oldColor;
	CRect rect;
	CGdiObject* pOldPen = NULL;
	CFont* oldFont;
	CString csWkday[7] = {" Sunday ", " Monday ", " Tuesday ", " Wednesday ",
		" Thursday ", " Friday ", " Saturday "};

	// Setup the page.
	Draw_Calendar(pDC);
	// If the Image needs to be behind the calendar, draw it now.
	if(!fGrafPos && !fGontop)
		Draw_Graphic(pDC);
	// Make the tracker rectangle
	L_margin = (int)(pDoc->ratio_left*dCW);
	T_margin = (int)(pDoc->ratio_top*dCH);
	nTW = (int)(pDoc->ratio_right*dCW-pDoc->ratio_left*dCW);
	nTW7 = nTW/7;
	R_margin = nTW+L_margin;
	int min = (int)((dCH*(pDoc->ratio_bottom-pDoc->ratio_top))/20);
	int max = (int)((nTW)/20);
	if(min < max)
		nwdheight = min;
	else
		nwdheight = max;
	// If either the date or the title should be large, multiple by 3.
	if(theApp.nDateLarge || pDoc->nTitleLarge)
		ntheight = nwdheight*3;
	else // Otherwise multiple by 1.5.
		ntheight = (int)(nwdheight*1.5);
	nTH = (int)((dCH*(pDoc->ratio_bottom-pDoc->ratio_top))-nwdheight-ntheight);
	nTH6 = nTH/6;
	B_margin = nTH+T_margin;
	// Set a rectangle, adding back the title box height and the weekday box height.
	rect.SetRect(L_margin, T_margin, R_margin, B_margin+nwdheight+ntheight);
	// Draw the rectangle.
	pDC->Rectangle(rect);
	// Copy our calculated rectangle into the calendar tracker rectangle to avoid discrepancies.
	m_calendar.m_rect.CopyRect(rect);
	// Show the tracker rectangle border if we are positioning
	if(fCalPos)
		m_calendar.Draw(pDC);
	// Make the title rectangle
	rect.SetRect(L_margin, T_margin, R_margin, T_margin+ntheight);
	// Draw the rectangle.
	pDC->Rectangle(rect);
	// Make a slightly smaller rectangle for the text.
	rect.DeflateRect((int)(rect.Width()*.01), (int)(rect.Height()*.08));
	// Draw the title and the date.
	Draw_Title_Date(pDC, rect, TRUE);
	// Make the grid
	for(col = 0; col < 7; col++)
	{
		// Make the weekday boxes
		nLeft = (col * nTW7)+L_margin;
		nRight = nLeft + nTW7+1;
		if(col == 6) nRight = nTW+L_margin;
		nTop = T_margin+ntheight-1;
		nBottom = T_margin+nwdheight+ntheight+1;
		rect.SetRect(nLeft, nTop, nRight, nBottom);
		// Draw the rectangle.
		pDC->Rectangle(rect);
		// Make a slightly smaller rectangle for the text.
		rect.DeflateRect((int)(rect.Width()*.05), (int)(rect.Height()*.15));
		if(col == 0)
		{
			// Calculate the Weekday font height.
			// Constants determined by trial and error.
			if((int)(rect.Width()/5.3) <= rect.Height())
				pDoc->lfWeekday.lfHeight = (int)(rect.Width()/5.3);
			else
				pDoc->lfWeekday.lfHeight = rect.Height();
			// Create the Weekday font using the document LOGFONT structure.
			Weekday.CreateFontIndirect(&pDoc->lfWeekday);
			// Save the current font.
			oldFont = pDC->GetCurrentFont();
			// Select the Weekday font.
			pDC->SelectObject(&Weekday);
			// Save the current text color.
			oldColor = pDC->SetTextColor(theApp.WkdayColor);
		}
		// Draw the weekday text
		pDC->DrawText(csWkday[col], rect, DT_SINGLELINE|DT_VCENTER|DT_CENTER);
		for(row = 0; row < 6; row++)
		{
			// Make the day boxes
			nLeft = (col * nTW7)+L_margin;
			nTop = (row * nTH6)+T_margin+nwdheight+ntheight;
			nRight = nLeft + nTW7+1;
			if(col == 6) nRight = nTW+L_margin;
			nBottom = nTop + nTH6+1;
			if(row == 5) nBottom = nTH+T_margin+nwdheight+ntheight;
			// Save the array of rectangles, one at a time.
			m_rect[col][row].SetRect(nLeft, nTop, nRight, nBottom);
		}
	}
	// Restore the old objects and delete the Weekday font we created.
	pDC->SelectObject(oldFont);
	pDC->SetTextColor(oldColor);
	Weekday.DeleteObject();
}

void CDaysEaseView::Print_Year(CDC * pDC, int print_left, int print_top,
		int print_right, int print_bottom)
{
	// Comments similar to View_Year() function.
	CMainFrame* pMain = theApp.Main_Pointer();
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	int nTop, nLeft, nRight, nBottom;
	int nTW, nTW7, nTH, nTH6;
	int L_margin, T_margin, R_margin, B_margin;
	int ntheight; // title box height
	char ch[5];
	itoa(pDoc->nYear, ch,10);
	CString csYear = ch;
	CRect rect;

	L_margin = print_left;
	T_margin = print_top;
	nTW = print_right-L_margin;
	int min = (print_bottom-T_margin)/20;
	int max = nTW/20;
	if(min < max)
		ntheight = min;
	else
		ntheight = max;
	if(theApp.nDateLarge || pDoc->nTitleLarge)
		ntheight = ntheight*3;
	else
		ntheight = (int)(ntheight*1.5);
	nTW7 = nTW/pMain->m_CalendarDlg.nYrx;
	R_margin = nTW+L_margin;
	nTH = print_bottom-T_margin-ntheight;
	nTH6 = nTH/pMain->m_CalendarDlg.nYry;
	B_margin = nTH+T_margin;
	// Make the title rectangle
	rect.SetRect(L_margin, T_margin, R_margin, T_margin+ntheight);
	pDC->Rectangle(rect);
	rect.DeflateRect((int)(rect.Width()*.01), (int)(rect.Height()*.08));
	Draw_Title_Date(pDC, rect, FALSE);
	// Make the grid
	int month = 0;
	for(row = 0; row < pMain->m_CalendarDlg.nYry; row++)
	{
		for(col = 0; col < pMain->m_CalendarDlg.nYrx; col++)
		{// Make the day boxes
			month++;
			nLeft = (col * nTW7)+L_margin;
			nTop = (row * nTH6)+T_margin+ntheight;
			nRight = nLeft + nTW7+1;
			if(col == pMain->m_CalendarDlg.nYrx-1) nRight = nTW+L_margin;
			nBottom = nTop + nTH6+1;
			if(row == pMain->m_CalendarDlg.nYry-1) nBottom = nTH+T_margin+ntheight;
			rect.SetRect(nLeft, nTop, nRight, nBottom);
			Mini_Month(pDC, rect, month, pDoc->nYear, rect.Height()/7);
		}
	}
}

void CDaysEaseView::Print_Month(CDC * pDC, int print_left, int print_top,
		int print_right, int print_bottom)
{
	// Comments similar to View_Month() function.
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	CFont Weekday;
	CFont* oldFont;
	COLORREF oldColor;
	int nwdheight; // weekday box height
	int ntheight; // title box height
	int nTop, nLeft, nRight, nBottom;
	int nTW, nTW7, nTH, nTH6;
	int L_margin, T_margin, R_margin, B_margin;
	CRect rect;
	CString csWkday[7] = {" Sunday ", " Monday ", " Tuesday ", " Wednesday ",
		" Thursday ", " Friday ", " Saturday "};

	L_margin = print_left;
	T_margin = print_top;
	nTW = print_right-L_margin;
	nTW7 = nTW/7;
	R_margin = nTW+L_margin;
	int min = (print_bottom-T_margin)/20;
	int max = nTW/20;
	if(min < max)
		nwdheight = min;
	else
		nwdheight = max;
	if(theApp.nDateLarge || pDoc->nTitleLarge)
		ntheight = nwdheight*3;
	else
		ntheight = (int)(nwdheight*1.5);
	nTH = (print_bottom-T_margin-nwdheight-ntheight);
	nTH6 = (nTH)/6;
	B_margin = nTH+T_margin;
	rect.SetRect(L_margin, T_margin, R_margin, B_margin+nwdheight+ntheight);
	pDC->Rectangle(rect);
	// Make the title rectangle
	rect.SetRect(L_margin, T_margin, R_margin, T_margin+ntheight);
	pDC->Rectangle(rect);
	rect.DeflateRect((int)(rect.Width()*.01), (int)(rect.Height()*.08));
	Draw_Title_Date(pDC, rect, TRUE);
	// Make the grid
	for(col = 0; col < 7; col++)
	{
		// Make the weekday boxes
		nLeft = (col * nTW7)+L_margin;
		nRight = nLeft + nTW7+1;
		if(col == 6) nRight = nTW+L_margin;
		nTop = T_margin+ntheight-1;
		nBottom = T_margin+nwdheight+ntheight+1;
		rect.SetRect(nLeft, nTop, nRight, nBottom);
		pDC->Rectangle(rect);
		rect.DeflateRect((int)(rect.Width()*.05), (int)(rect.Height()*.15));
		if(col == 0)
		{
			if((int)(rect.Width()/5.3) <= rect.Height())
				pDoc->lfWeekday.lfHeight = (int)(rect.Width()/5.3);
			else
				pDoc->lfWeekday.lfHeight = rect.Height();
			Weekday.CreateFontIndirect(&pDoc->lfWeekday);
			oldFont = pDC->GetCurrentFont();
			pDC->SelectObject(&Weekday);
			oldColor = pDC->SetTextColor(theApp.WkdayColor);
		}
		// Insert the weekday text
		pDC->DrawText(csWkday[col], rect, DT_SINGLELINE|DT_VCENTER|DT_CENTER);
		for(row = 0; row < 6; row++)
		{// begin for loop
			nLeft = (col * nTW7)+L_margin;
			nTop = (row * nTH6)+T_margin+nwdheight+ntheight;
			nRight = nLeft + nTW7+1;
			if(col == 6) nRight = nTW+L_margin;
			nBottom = nTop + nTH6+1;
			if(row == 5) nBottom = nTH+T_margin+nwdheight+ntheight;
			m_rect[col][row].SetRect(nLeft, nTop, nRight, nBottom);
		}
	}
	pDC->SelectObject(oldFont);
	pDC->SetTextColor(oldColor);
	Weekday.DeleteObject();
}

void CDaysEaseView::Insert_Day_Number(CDC * pDC, CRect * pRect, int k, int nDays)
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	char chDay[2];
	CRect rect;
	CString csDocData, csDay;
	CString eol = CString(0x0d) + CString(0x0a);
	CPen* pOldPen;
	CGdiObject* pOldBrush;
	CFont Weekday, Notes;
	CFont* oldFont;
	COLORREF oldColor;
	CString string, text(" ");
	int min_size;
	
	fNote_Warn = FALSE;
	
	// Convert the day integer to a string
	itoa(k, chDay, 10);
	csDay = chDay;
	// Save the current font.
	oldFont = pDC->GetCurrentFont();
	// Create the Weekday font using the document LOGFONT structure.
	Weekday.CreateFontIndirect(&pDoc->lfWeekday);
	// Select the Weekday font.
	pDC->SelectObject(&Weekday);
	// Save the current text color.
	oldColor = pDC->SetTextColor(theApp.WkdayColor);
	// Get any text from the document day specfied by col and row.
	csDocData = pDoc->GetSquare (col, row);
	// Depending on whether the day number has 1 or 2 characters,
	// strip it of and save the rest of the text.
	if(csDocData.Left(csDay.GetLength()) == csDay)
		csDocData = csDocData.Mid(csDay.GetLength());
	// If the day number is preceded by a tab character,
	// do the same.
	if(csDocData.Left(csDay.GetLength() + 1) == (char)0x09 + csDay)
		csDocData = csDocData.Mid(csDay.GetLength() + 1);
	// If the event flag is set, go get the events for this day.
	if(fEvents)
	{
		string = Insert_Events(csDay, csDocData, (k -1)/7, (k + 7) > nDays);
		// If events exist for this day, save them.
		// This is the only time we insert text into the m_Eventgrid array.
		if(!string.IsEmpty())
			theApp.m_Eventgrid[col][row] = eol + string;
	}
	// If we have no text for this day...
	if(csDocData.IsEmpty())
	{
		// We need an end of line to separate from events.
		csDocData = eol;
		// Update the document with just the day number on its own line.
		pDoc->Add_Data(csDay + eol, col, row);
	}
	else // If we have text for this day...
	{
		// Make sure the day number and text are separated by an end of line.
		if(csDocData.Left(2) != eol)
			csDocData = eol + csDocData;
		// Update the document with the day number on its own line followed by the text.
		pDoc->Add_Data(csDay + csDocData, col, row);
	}
	// Make a string with the day text and the day events.
	string = csDocData + theApp.m_Eventgrid[col][row];
	// Create a margin for the screen text
	rect.CopyRect(pRect);
	// The margin values were found by trial and error.
	rect.DeflateRect((int)(rect.Width()*.07), (int)(rect.Height()*.06),
		(int)(rect.Width()*.07), (int)(rect.Height()*.02));
	// Draw the day number using the Weekday font.
	pDC->DrawText(csDay, rect, theApp.nDt_Swap|DT_EDITCONTROL|DT_NOPREFIX|DT_WORDBREAK);

	// Set the initial Day Notes font size.
	int W9 = (int)(rect.Width()/9);
	int H9 = (int)(rect.Height()/9);
	if(theApp.nNoteLarge)
	{
		if(W9 <= H9)
			pDoc->lfNotes.lfHeight = (int)(rect.Width()/7);
		else
			pDoc->lfNotes.lfHeight = (int)(rect.Height()/7);
	}
	else
	{
		if(W9 <= H9)
			pDoc->lfNotes.lfHeight = W9;
		else
			pDoc->lfNotes.lfHeight = H9;
	}
	// Set a minimum size.
	if(pDC->IsPrinting())
	{
		if(W9 <= H9)
			min_size = (int)(W9*.9);
		else
			min_size = (int)(H9*.9);
	}
	else
	{
		if(W9 <= H9)
			min_size = W9;
		else
			min_size = H9;
	}
	// Create the Notes font from the modified document LOGFONT structure.
	Notes.CreateFontIndirect(&pDoc->lfNotes);
	// Select the Notes font.
	pDC->SelectObject(&Notes);
	// Set the text color.
	pDC->SetTextColor(theApp.NotesColor);
	// Shortten the rectangle so we don't overwrite the day number.
	rect.DeflateRect(0, (int)(rect.Height()*.1), 0, 0);
	// Create a dummy rectangle.
	CRect dummy;
	// Make it the same size as this one.
	dummy.CopyRect(rect);
	// Test whether the text will fit in the dummy rectangle.
	int height = pDC->DrawText(string, dummy, DT_CALCRECT|DT_EDITCONTROL|DT_NOPREFIX|DT_WORDBREAK);
	// If it didn't fit, try making the font smaller until it does.
	while((height > rect.Height()) && (pDoc->lfNotes.lfHeight > min_size))
	{
		// Select the oldfont, so we can delete the Notes font.
		pDC->SelectObject(oldFont);
		Notes.DeleteObject();
		// Decrement the Notes font height.
		pDoc->lfNotes.lfHeight--;
		//Recreate the Notes font with the smaller size.
		Notes.CreateFontIndirect(&pDoc->lfNotes);
		// Select the Notes font.
		pDC->SelectObject(&Notes);
		// Reset the dummy rectangle.
		dummy.CopyRect(rect);
		// See if the new Notes font size fits.
		height = pDC->DrawText(string, dummy, DT_CALCRECT|DT_EDITCONTROL|DT_NOPREFIX|DT_WORDBREAK);
	}
	// If we were unable to make the text fit, set the warning flag.
	if(height > rect.Height())
		fNote_Warn++;
	// Draw the text string on the screen.
	pDC->DrawText(string, rect, DT_LEFT|DT_EDITCONTROL|DT_NOPREFIX|DT_WORDBREAK);
	// If the warning flag is set and we are not printing, draw a violet border around the day box
	if(fNote_Warn && !pDC->IsPrinting())
	{
		rect.CopyRect(pRect);
		rect.DeflateRect(2, 2, 1, 1);
		CPen pen (PS_SOLID, 2, RGB(255, 0, 255));
		pOldPen = pDC->SelectObject (&pen);
		pOldBrush = pDC->SelectStockObject(NULL_BRUSH);
		pDC->Rectangle(rect);
		pDC->SelectObject(pOldPen);
		pDC->SelectObject(pOldBrush);
	}
	// If this rectangle represents the currently selected day and we are not printing
	// draw a border around the day box.
	if(m_rect[nCurX][nCurY].EqualRect(pRect) && !pDC->IsPrinting())
	{// Draw a border around the current day
		CPen pen1(PS_SOLID, 2, RGB(255, 0, 0));
		CPen pen2(PS_SOLID, 2, RGB(0, 0, 255));
		rect.CopyRect(pRect);
		rect.DeflateRect(2, 2, 1, 1);
		// If the warning flag is set, draw a red border.
		if(fNote_Warn)
			pOldPen = pDC->SelectObject (&pen1);
		else // Otherwise, draw a blue border.
			pOldPen = pDC->SelectObject (&pen2);
		pOldBrush = pDC->SelectStockObject(NULL_BRUSH);
		pDC->Rectangle(rect);
		pDC->SelectObject(pOldPen);
		pDC->SelectObject(pOldBrush);
	}
	// Select our old objects so we can delete those we created to prevent memory leaks.
	pDC->SetTextColor(oldColor);
	pDC->SelectObject(oldFont);
	Weekday.DeleteObject();
	Notes.DeleteObject();
}


void CDaysEaseView::Mini_Month(CDC * pDC, CRect rect, int month, int year, int X)
{
	// The following two lines were added in version 1.1 to exclude years prior to 1970 and after 2037.
	// Found by Dennis Cummins.
	if((year < 1970) || (year > 2037))
		return;
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	int nTop, nLeft, nRight, nBottom;
	int nTW, nTW7, nTH, nTH6;
	int L_margin, T_margin, R_margin, B_margin;
	int col, row;
	int notesize;
	char ch[5];
	int nDays = 0;
	CFont Weekday, Notes;
	CFont* oldFont;
	COLORREF oldColor;
	CTimeSpan cts;
	CTime ctCurrent_Date, ctFirst_Day, ctLast_Day;
	CString csMonth[12] = {"January", "February", "March", "April", "May",
		"June", "July", "August", "September", "October", "November", "December"};

	// Calculate a size for the Weekday font.
	if(rect.Width() <= rect.Height())
		pDoc->lfWeekday.lfHeight = rect.Width()/8;
	else
		pDoc->lfWeekday.lfHeight = rect.Height()/8;
	// Create the Weekday font from the document LOGFONT structure.
	Weekday.CreateFontIndirect(&pDoc->lfWeekday);
	// Save the current font.
	oldFont = pDC->GetCurrentFont();
	// Select the Weekday font.
	pDC->SelectObject(&Weekday);
	// Save the current text color.
	oldColor = pDC->SetTextColor(theApp.WkdayColor);
	// Get a CTime object for the first day of this month
	ctFirst_Day = CTime(year, month, 1, 0, 0, 0, 0);
	// If this is December, increment year and set next month to 1
	if(month == 12)
		ctLast_Day = CTime(year + 1, 1, 1, 0, 0, 0, 0);
	else // Otherwise increment month only
		ctLast_Day = CTime(year, month + 1, 1, 0, 0, 0, 0);
	// Get a CTimeSpan value for the month
	// this method frees us from calculating leap years
	// because CTimeSpan does it automatically
	cts = ctLast_Day - ctFirst_Day;
	// Get the number of days for this month
	nDays = cts.GetDays();
	// Draw the rectangle passed to this function.
	pDC->Rectangle(rect);
	// Make a rectangle a little smaller for the text.
	rect.DeflateRect((int)(rect.Width()*.02), (int)(rect.Height()*.03));
	// Draw the text in the rectangle.
	pDC->DrawText(csMonth[month-1], rect, DT_SINGLELINE|DT_CENTER|DT_TOP);
	// Save the current Notes font size.
	notesize = pDoc->lfNotes.lfHeight;
	// Calculate a new Notes font size.
	if(rect.Width() <= rect.Height())
		pDoc->lfNotes.lfHeight = rect.Width()/10;
	else
		pDoc->lfNotes.lfHeight = rect.Height()/10;
	// Create the Notes font.
	Notes.CreateFontIndirect(&pDoc->lfNotes);
	// Select the Notes font.
	pDC->SelectObject(&Notes);
	// Set the text color.
	pDC->SetTextColor(theApp.NotesColor);
	// Calculate the box size that we'll use to insert day numbers.
	L_margin = rect.left;
	T_margin = rect.top;
	nTW = rect.right-rect.left;
	nTW7 = nTW/7;
	nTW = nTW7*7;
	R_margin = nTW+L_margin;
	nTH = rect.bottom-rect.top-X;
	nTH6 = nTH/6;
	nTH = nTH6*6;
	B_margin = nTH+T_margin;
	
	int k = 0;
	// Draw the first week starting with the first day
	for (col = ctFirst_Day.GetDayOfWeek() - 1; col < 7; col++)
	{
		k++;
		nLeft = (col*nTW7)+L_margin;
		nTop = T_margin+X;
		nRight = nLeft + nTW7+1;
		if(col == 6) nRight -= 1;
		nBottom = nTop + nTH6+1;
		rect.SetRect(nLeft, nTop, nRight, nBottom);
		itoa(k, ch, 10);
		CString d = ch;
		// Draw the day number in the box.
		pDC->DrawText(d, rect, DT_SINGLELINE|DT_CENTER|DT_VCENTER);
	}
	// Create the remaining days until the end of the month
	for (row = 1; row < 6 && k < nDays; row++)
	{
		for (col = 0; col < 7 && k < nDays; col++)
		{
			k++;
			nLeft = (col * nTW7)+L_margin;
			nTop = (row * nTH6)+T_margin+X;
			nRight = nLeft + nTW7+1;
			if(col == 6) nRight -= 1;
			nBottom = nTop + nTH6+1;
			if(row == 5) nBottom -= 1;
			rect.SetRect(nLeft, nTop, nRight, nBottom);
			itoa(k, ch, 10);
			CString d = ch;
			// Draw the day number in the box.
			pDC->DrawText(d, rect, DT_SINGLELINE|DT_CENTER|DT_VCENTER);
		}
	}
	// Restore the Note font size.
	pDoc->lfNotes.lfHeight = notesize;
	// Select our old objects so we can delete the ones we created to prevent memory leaks.
	pDC->SetTextColor(oldColor);
	pDC->SelectObject(oldFont);
	Weekday.DeleteObject();
	Notes.DeleteObject();
}

void CDaysEaseView::Draw_Title_Date(CDC * pDC, CRect rect, BOOL trufal)
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	CMainFrame* pMain = theApp.Main_Pointer();
	// Adjustment factor derived from trial and error.
	int x = 7;
	CString csMon, csYr;
	UINT DT_FIRST = DT_LEFT;
	UINT DT_SECOND = DT_RIGHT;
	UINT DT_TITLE = DT_LEFT;
	COLORREF oldColor;
	CFont Date, Title;
	CFont* oldFont;

	// Calculate the font sizes for date and title.
	if((pDoc->nTitleLarge & theApp.nDateLarge) ||
		(!pDoc->nTitleLarge & !theApp.nDateLarge))
	{
		if(rect.Height() < rect.Width()/x)
		{
			pDoc->lfTitle.lfHeight = rect.Height()-1;
			pDoc->lfDate.lfHeight = rect.Height()-1;
		}
		else
		{
			pDoc->lfTitle.lfHeight = rect.Width()/x;
			pDoc->lfDate.lfHeight = rect.Width()/x;
		}
	}
	if(!pDoc->nTitleLarge & theApp.nDateLarge)
	{
		if(rect.Height() < rect.Width()/x)
		{
			pDoc->lfTitle.lfHeight = rect.Height()/2;
			pDoc->lfDate.lfHeight = rect.Height();
		}
		else
		{
			pDoc->lfTitle.lfHeight = rect.Width()/(2*x);
			pDoc->lfDate.lfHeight = rect.Width()/x;
		}
	}
	if(pDoc->nTitleLarge & !theApp.nDateLarge)
	{
		if(rect.Height() < rect.Width()/x)
		{
			pDoc->lfTitle.lfHeight = rect.Height();
			pDoc->lfDate.lfHeight = rect.Height()/2;
		}
		else
		{
			pDoc->lfTitle.lfHeight = rect.Width()/x;
			pDoc->lfDate.lfHeight = rect.Width()/(2*x);
		}
	}
	// Create the Title and Date fonts using the calculated sizes and the document LOGFONT structures.
	Title.CreateFontIndirect(&pDoc->lfTitle);
	Date.CreateFontIndirect(&pDoc->lfDate);
	// Save the current font.
	oldFont = pDC->GetCurrentFont();
	// Select the Date font.
	pDC->SelectObject(&Date);
	// Save the current text color.
	oldColor = pDC->SetTextColor(theApp.DateColor);
	// Update our public title rectangle size.
	crTitle = rect;
	// Get the title text from the Style page.
	pMain->m_StyleDlg.m_Title.GetWindowText(pDoc->cstitle);
	// Update the Style page buttons as needed.
	if(pDoc->cstitle.IsEmpty())
	{
		pMain->m_StyleDlg.m_btnTitle.EnableWindow(FALSE);
		pMain->m_StyleDlg.m_TitleLarge.EnableWindow(FALSE);
	}
	else
	{
		pMain->m_StyleDlg.m_btnTitle.EnableWindow(TRUE);
		pMain->m_StyleDlg.m_TitleLarge.EnableWindow(TRUE);
	}
	// If this is a moth calendar, get the month.
	if(trufal)
		pMain->m_CalendarDlg.m_Month.GetWindowText(csMon);
	// Get the year in either case.
	pMain->m_CalendarDlg.m_Year.GetWindowText(csYr);
	// Determine the positions of the text.
	if(theApp.nDt_Custom == -1)
	{
		if(theApp.nDateSwap)
		{
			// If italic font, we need to pad with spaces so it doesn't truncate.
			if(pDoc->lfDate.lfItalic != NULL)
			{
				csMon += " ";
				csYr = " " + csYr;
			}
			DT_FIRST = DT_RIGHT;
			DT_SECOND = DT_LEFT;
		}
		else // If italic font, we need to pad with spaces so it doesn't truncate.
			if(pDoc->lfDate.lfItalic != NULL)
			{
				csYr += " ";
				csMon = " " + csMon;
			}
		// Draw the month and year text.
		pDC->DrawText(csMon, rect, DT_SINGLELINE|DT_VCENTER|DT_FIRST);
		pDC->DrawText(csYr, rect, DT_SINGLELINE|DT_VCENTER|DT_SECOND);
		// Set the text color.
		pDC->SetTextColor(theApp.TitleColor);
		// Select the Title font.
		pDC->SelectObject(&Title);
		// Draw the title.
		pDC->DrawText(pDoc->cstitle, rect, DT_SINGLELINE|DT_VCENTER|DT_CENTER);
	}
	else
	{
		CString text = csYr;

		if(trufal && !theApp.nDateSwap)
			text = csMon + " " + csYr;
		if(trufal && theApp.nDateSwap)
			text = csYr + " " + csMon;
		// If italic font, we need to pad with spaces so it doesn't truncate.
		if(theApp.nDt_Custom == DT_RIGHT && pDoc->lfDate.lfItalic != NULL)
			text += " ";
		// If italic font, we need to pad with spaces so it doesn't truncate.
		if(theApp.nDt_Custom == DT_LEFT && pDoc->lfDate.lfItalic != NULL)
			text = " " + text;
		pDC->DrawText(text, rect, DT_SINGLELINE|DT_VCENTER|theApp.nDt_Custom);
		text = pDoc->cstitle;
		if(theApp.nDt_Custom == DT_LEFT)
		{
			// If italic font, we need to pad with spaces so it doesn't truncate.
			if(pDoc->lfTitle.lfItalic != NULL)
				text += " ";
			DT_TITLE = DT_RIGHT;
		}
		// If italic font, we need to pad with spaces so it doesn't truncate.
		if(theApp.nDt_Custom == DT_RIGHT && pDoc->lfTitle.lfItalic != NULL)
				text = " " + text;
		// Set the text color.
		pDC->SetTextColor(theApp.TitleColor);
		// Select the Title font.
		pDC->SelectObject(&Title);
		// Draw the title.
		pDC->DrawText(text, rect, DT_SINGLELINE|DT_VCENTER|DT_TITLE);
	}
	// Select our old objects so we can delete the ones we created to prevent memory leaks.
	pDC->SetTextColor(oldColor);
	pDC->SelectObject(oldFont);
	Title.DeleteObject();
	Date.DeleteObject();
}

CString CDaysEaseView::Insert_Events(CString D, CString csDocData, int week, BOOL last)
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	CString M, Y, W, WD, string("");
	CString csMonth[12] = {"Jan", "Feb", "Mar", "Apr", "May",
		"Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
	CString csWeek[5] = {"1", "2", "3", "4", "L"};
	CString csWeekDay[7] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
	char ch[5];

	// Make sure the day number is two digits.
	if(D.GetLength() < 2)
		D = "0" + D;
	// Set M = current month abbreviation.
	M = csMonth[pDoc->nMonth - 1];
	// Set WD = current week day abbreviation, which happens to correspond to the current column.
	WD = csWeekDay[col];
	// Set W = string corresponding to week integer.
	W = csWeek[week];
	// Convert current year integer to string Y.
	itoa(pDoc->nYear, ch, 10);
	Y = ch;
	// Run through the various event formats letting Find_Event() return the caption strings.
	string += Find_Event(M + " " + D, csDocData);
	string += Find_Event(M + " " + D + ", " + Y, csDocData);
	string += Find_Event(M + "-" + WD + " #" + W, csDocData);
	// Fourth week could also be last.
	if(last && W == "4")
		string += Find_Event(M + "-" + WD + " #L", csDocData);
	return string;
}

CString CDaysEaseView::Find_Event(CString look, CString csDocData)
{
	CString EventList, temp_data, string("");
	CString eol = CString(0x0d) + CString(0x0a);

	// Add a tab character to the date abbreviation.
	look += (char)0x09;
	// Copy the Event List.
	EventList = theApp.csEventList;
	// Look for events starting with this date abbreviation + tab character.
	// Note, by including the tab character in the search, we eliminate the possibility
	// of finding a date abbreviation within an event caption.
	while(EventList.Find(look) != -1)
	{
		// Remove all but the caption.
		EventList = EventList.Mid(EventList.Find(look));
		EventList = EventList.Mid(EventList.Find((char)0x09) +1);
		temp_data = EventList.Mid(0, EventList.Find(eol) +2);
		if(csDocData.Find(eol + temp_data) == -1)
			string += temp_data;
	}
//delit	if(look == "Apr 4")
//		look = look;
	// If this date abbreviation is in our Easter calculated string
	// and EASTER is in the Event List, then we have a match.
	if(csEaster.Find(look) != -1 &&
			theApp.csEventList.Find(CString("EASTER") + (char)0x09) != -1)
		string += "Easter" + eol;
	return string;
}

void CDaysEaseView::Draw_Calendar(CDC* pDC)
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	CBrush yelbrush(RGB(255,255,0));
	CGdiObject* pOldBrush;
	CGdiObject* pOldPen;
	CRect rect, crClient;
	double margin_vert;
	double margin_horiz;
	double dPageSize;

	// Create the margin ratios.  cxPage and cyPage represent the width and height
	// of the printable area. physoffsetx and physoffsety represent the left and top
	// margins.  physwidth and physheight are the total width and height of the page.
	// These values are reported by the printer and retrieved in Get_Printer_Stats().

	// Some printer drivers may increase the left margin instead of the right,
	// so check and reverse if needed.  Note; include a fudge factor, 10,
	// to offset the fact that we reduce cxPage by 10.

	// The following three lines were added in version 1.1 to fix a bug where some printers would not center
	// the page if the left margin was greater than the right margin.  Found by Terry Farr.
	if(physoffsetx > (physwidth - cxPage - physoffsetx - 10))
		margin_horiz = (physwidth - physoffsetx - 10)/(double)physwidth;
	else
		margin_horiz = (cxPage + physoffsetx)/(double)physwidth;

	// Some printer drivers may increase the top margin instead of the bottom,
	// so check and reverse if needed.  Note; include a fudge factor, 10,
	// to offset the fact that we reduce cyPage by 10.
	if(physoffsety > (physheight - cyPage - physoffsety - 10))
		margin_vert = (physheight - physoffsety - 10)/(double)physheight;
	else
		margin_vert = (cyPage + physoffsety)/(double)physheight;
	// If we have set ratio_top to -1, then set the calendar ratios to full page.
	if(pDoc->ratio_top == -1)
	{
		// Since margin_horiz represents the right margin, 1-margin_horiz represents
		// the left margin.  1 refers to one unit, whether it's the client width or the
		// the full page width.
		pDoc->ratio_left = 1-margin_horiz;
		pDoc->ratio_right = margin_horiz;
		// Since margin_vert represents the bottom margin, 1-margin_vert represents
		// the top margin.  1 refers to one unit, whether it's the client height or the
		// the full page height.
		pDoc->ratio_top = 1-margin_vert;
		pDoc->ratio_bottom = margin_vert;
	}
	// If we haven't set ratio_top to -1, then check and correct the ratios to be sure
	// that they haven't exceeded the margins.
	else
	{
		if(pDoc->ratio_top < 1-margin_vert)
			pDoc->ratio_top = 1-margin_vert;
		if(pDoc->ratio_bottom > margin_vert)
			pDoc->ratio_bottom = margin_vert;
		if(pDoc->ratio_left < 1-margin_horiz)
			pDoc->ratio_left = 1-margin_horiz;
		if(pDoc->ratio_right > margin_horiz)
			pDoc->ratio_right = margin_horiz;
		// If we have a graphic image, do the same for it.
		if(pDoc->fGraphic)
		{
			if(pDoc->gratio_top < 1-margin_vert)
				pDoc->gratio_top = 1-margin_vert;
			if(pDoc->gratio_bottom > margin_vert)
				pDoc->gratio_bottom = margin_vert;
			if(pDoc->gratio_left < 1-margin_horiz)
				pDoc->gratio_left = 1-margin_horiz;
			if(pDoc->gratio_right > margin_horiz)
				pDoc->gratio_right = margin_horiz;
		}
	}
	GetClientRect(crClient);
	// If we are in landscape orientation, then dPageSize is height divided by width.
	if(theApp.Orientation == DMORIENT_LANDSCAPE)
	{
		dPageSize = (double)physheight/(double)physwidth;
		// Width is one unit, so set it equal to the client width.
		dCW = crClient.Width();
		// Height is one unit times the dPageSize ratio.
		dCH = dPageSize*dCW;
		// Make a rectangle for the unused lower client area.
		rect.SetRect(0, (int)dCH,
			(int)dCW, 2000);// Bottom set to arbitrarily large value
	}
	// Otherwise we are in portrait orientation, so dPageSize is width divided by height.
	else
	{
		dPageSize = (double)physwidth/(double)physheight;
		// Height is one unit, so set it equal to the client height.
		dCH = crClient.Height();
		// Width is one unit times the dPageSize ratio.
		dCW = dPageSize*dCH;
		// Make a rectangle for the unused right client area.
		rect.SetRect((int)dCW, 0,
			2000, (int)dCH);// Right set to arbitrarily large value
	}
	// Make unused client area gray
	pOldPen = pDC->SelectStockObject(NULL_PEN);
	pOldBrush = pDC->SelectStockObject(LTGRAY_BRUSH);
	pDC->Rectangle(rect);
	// Define the usable page area rectangle.
	theApp.page_rect.SetRect((int)((1-margin_horiz)*dCW), (int)((1-margin_vert)*dCH),
		(int)(margin_horiz*dCW), (int)(margin_vert*dCH));
	// Draw the unusable page area yellow.  We could make the entire client area yellow
	// and then draw the unused gray area and over write the usable page in white,
	// but that causes an annoying yellow flash.  Instead, draw each yellow border separately.
	pDC->SelectObject(&yelbrush);
	if(theApp.Orientation == DMORIENT_LANDSCAPE)
		rect.SetRect(0, 0, (int)dCW, theApp.page_rect.top);
	else
		rect.SetRect(0, 0, theApp.page_rect.right, theApp.page_rect.top);
	pDC->Rectangle(rect);
	rect.right = theApp.page_rect.left;
	rect.bottom = (int)dCH;
	pDC->Rectangle(rect);
	rect.left = theApp.page_rect.right;
	rect.right = (int)dCW;
	pDC->Rectangle(rect);
	rect.left = 0;
	rect.top = theApp.page_rect.bottom;
	pDC->Rectangle(rect);
	// Replace our brush and pen with originals.
	pDC->SelectObject(pOldBrush);
	pDC->SelectObject(pOldPen);
}

void CDaysEaseView::OnFilePrint()
{
	// Make sure the print dialog structure contains our current variables.
	m_pPrintDialog->GetDevMode()->dmOrientation = theApp.Orientation;
	m_pPrintDialog->GetDevMode()->dmPaperSize = theApp.PaperSize;
	strcpy((char*)(m_pPrintDialog->GetDevMode()->dmDeviceName), theApp.PrinterName);
	m_pPrintDialog->GetDevMode()->dmColor = theApp.PrintColor;

	if(IDOK == m_pPrintDialog->DoModal())
	{
		// Get stats in case user changed any parameters.
		Get_Printer_Stats(m_pPrintDialog->m_pd.hDC);
		// Save any changes and set modified flag if so.
		if(Save_Printer_Changes(TRUE))
			theApp.Set_Modified_Flag();
		// Reset the printer name in the statusbar in case we have a new printer.
		Set_Status_Text(1, theApp.PrinterName);
		// Reset the orientation button in case the user changed paper orientation.
		if(theApp.Orientation == DMORIENT_PORTRAIT)
			theApp.Main_Pointer()->m_CalendarDlg.m_Orientation.SetWindowText("Landscape");
		else
			theApp.Main_Pointer()->m_CalendarDlg.m_Orientation.SetWindowText("Portrait");
		// Update the View.
		Invalidate();
		// Now print the calendar.
		CView::OnFilePrint();
	}
}

BOOL CDaysEaseView::Save_Printer_Changes(BOOL new_setting)
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	BOOL trufal(0);

	if(new_setting && (pDoc->PrinterName != theApp.PrinterName))
	{
		// If we have new settings and the document shows a different printer, change the document.
		pDoc->PrinterName = theApp.PrinterName;
		trufal++;
		// Save the new info in DaysEase.ini.
		theApp.WriteProfileString("DaysEase", "Printer", theApp.PrinterName);
	}
	if(pDoc->Orientation != theApp.Orientation)
	{
		if(new_setting)
			pDoc->PrinterName = theApp.PrinterName;
		else
			theApp.Orientation = pDoc->Orientation;
		// Update our print dialog.
		m_pPrintDialog->GetDevMode()->dmOrientation = theApp.Orientation;
		trufal++;
		// Save the new info in DaysEase.ini.
		theApp.WriteProfileInt(theApp.PrinterName, "Orientation", theApp.Orientation);
	}
	if(pDoc->PaperSize != theApp.PaperSize)
	{
		if(new_setting)
			pDoc->PaperSize = theApp.PaperSize;
		else
			theApp.PaperSize = pDoc->PaperSize;
		// Update our print dialog.
		m_pPrintDialog->GetDevMode()->dmPaperSize = theApp.PaperSize;
		trufal++;
		// Save the new info in DaysEase.ini.
		theApp.WriteProfileInt(theApp.PrinterName, "Paper Size", theApp.PaperSize);
	}
	if(pDoc->PrintColor != theApp.PrintColor)
	{
		if(new_setting)
			pDoc->PrintColor = theApp.PrintColor;
		else
			theApp.PrintColor = pDoc->PrintColor;
		// Update our print dialog.
		m_pPrintDialog->GetDevMode()->dmColor = theApp.PrintColor;
		trufal++;
		// Save the new info in DaysEase.ini.
		theApp.WriteProfileInt(theApp.PrinterName, "Print Color", theApp.PrintColor);
	}
	// If anything was saved, then trufal is TRUE.
	return trufal;
}

BOOL CDaysEaseView::Validate_Printer()
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	DWORD pcbNeeded, pcReturned;
	PRINTER_INFO_2* pPI;
	BOOL trufal(0);

	// If the document specifies a printer, see if it exists on this machine.
	if(pDoc->PrinterName != "")
	{
		// First call EnumPrinters() with no PRINTER_INFO structure to obtain the character bytes needed.
		EnumPrinters(PRINTER_ENUM_LOCAL, NULL, 2, (LPBYTE)"",
			0, &pcbNeeded, &pcReturned);
		// Allocate memory for the PRINTER_INFO structure.
		pPI = (PRINTER_INFO_2*)HeapAlloc(GetProcessHeap(), 0, pcbNeeded);
		// Now call EnumPrinters() to fill in the PRINTER_INFO structure.
		if(EnumPrinters(PRINTER_ENUM_LOCAL, NULL, 2, (LPBYTE)pPI,
			pcbNeeded, &pcbNeeded, &pcReturned))
		{
			// Step through the PRINTER_INFO structure to see if it contains the document printer.
			for(int i = 0; i < (int)pcReturned; i++)
			{
				// If found, set trufal.
				if((pPI + i)->pPrinterName == pDoc->PrinterName)
					trufal++;
			}
		}
		// Always free the allocated heap memory to prevent leaks.
		HeapFree(GetProcessHeap(), 0, pPI);
	}
	return trufal;
}

void CDaysEaseView::OnSetup() 
{
	// Set the print dialog to Printer Setup.
	m_pPrintDialog->m_pd.Flags |= PD_PRINTSETUP;
	// Make sure the print dialog structure contains our current variables.
	m_pPrintDialog->GetDevMode()->dmOrientation = theApp.Orientation;
	m_pPrintDialog->GetDevMode()->dmPaperSize = theApp.PaperSize;
	m_pPrintDialog->GetDevMode()->dmColor = theApp.PrintColor;
	if(IDOK == m_pPrintDialog->DoModal())
	{
		// Get stats in case user changed any parameters.
		Get_Printer_Stats(m_pPrintDialog->m_pd.hDC);
		// Save any changes and set modified flag if so.
		if(Save_Printer_Changes(TRUE))
			theApp.Set_Modified_Flag();
		// Reset the printer name in the statusbar in case we have a new printer.
		Set_Status_Text(1, theApp.PrinterName);
		// Reset the orientation button in case the user changed paper orientation.
		if(theApp.Orientation == DMORIENT_PORTRAIT)
			theApp.Main_Pointer()->m_CalendarDlg.m_Orientation.SetWindowText("Landscape");
		else
			theApp.Main_Pointer()->m_CalendarDlg.m_Orientation.SetWindowText("Portrait");
		// Update the View.
		Invalidate();
	}
	// Remove the Printer Setup flag from the print dialog.
	m_pPrintDialog->m_pd.Flags ^= PD_PRINTSETUP;
}

BOOL CDaysEaseView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	// If we are positioning the calendar or the Image, and the pointer is over one of them,
	// interupt and let CRectTracker handle the mouse cursor.
	if(fCalPos)
	{
		if(pWnd == this && m_calendar.SetCursor(this, nHitTest))
			return TRUE;
	}
	if(fGrafPos)
	{
		if(pWnd == this && m_graphic.SetCursor(this, nHitTest))
			return TRUE;
	}
	
	return CView::OnSetCursor(pWnd, nHitTest, message);
}

void CDaysEaseView::OnTimer(UINT nIDEvent) 
{
	CMainFrame* pMain = theApp.Main_Pointer();
	CPoint point;
	CRect rect;


	// If the mouse pointer is not within the page rectangle screen coordinates,
	// kill the timer so the statusbar can change.
	GetCursorPos(&point);
	GetWindowRect(rect);
	rect.left += 2;
	rect.right = rect.left + (int)dCW;
	rect.bottom = rect.top + (int)dCH + 3;
	if(!rect.PtInRect(point))
	{
		// Hide our "Test" edit box.
		theApp.Main_Pointer()->m_CalendarDlg.m_TextCheck.ShowWindow(SW_HIDE);
		KillTimer(nIDEvent);
		if(!fCalPos && !fGrafPos)
		{
			if(pMain->m_CalendarDlg.m_Data.IsWindowVisible())
				Set_Status_Text(0, "Right click any Day Box to see all text");
			else
				Set_Status_Text(0, "");
		}
	}
	CView::OnTimer(nIDEvent);
}

void CDaysEaseView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// Set the statusbar for user reminders.

	// The following line was changed in version 1.1 so that the code would compile without a warning in VC++ 6.0
	// Found by Gary Johnson.
	//if(point.x <= dCW & point.y <= dCH)
	if((point.x <= dCW) && (point.y <= dCH))
	{
		CRect rectSave;

		if(theApp.Doc_Pointer()->fGraphic)
		{
			m_graphic.GetTrueRect(rectSave);
			if(rectSave.PtInRect(point))
				if(!fCalPos && !fGrafPos)
					Set_Status_Text(0, "Double click graphic to position");
		}
		m_calendar.GetTrueRect(rectSave);
		if(rectSave.PtInRect(point))
			if(!fCalPos && !fGrafPos)
				Set_Status_Text(0, "Double click calendar to position");
		// Set a timer to blank the statusbar after 1/2 second.
		// Since we only have one timer, we don't need to save the return ID value.
		SetTimer(1, 500, NULL);
	}
	CView::OnMouseMove(nFlags, point);
}
// Handle Left mouse button double clicks.
void CDaysEaseView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	if(fGontop)
	{
		if(!Img_Hit_Test(point))
			Cal_Hit_Test(point);
	}
	else
	{
		if(!Cal_Hit_Test(point))
			Img_Hit_Test(point);
	}
	CView::OnLButtonDblClk(nFlags, point);
}

void CDaysEaseView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();

	// Handle Image position clicks.
	if(fGrafPos)
	{
		if(m_graphic.HitTest(point) == -1)
		{
			Set_Status_Text(0, "");
			fGrafPos = 0;
		}
		else
			Position_Graphic(point);
		Invalidate();
		CView::OnLButtonDown(nFlags, point);
		return;
	}
	// Handle Calendar position clicks.
	if(fCalPos)
	{
		if(m_calendar.HitTest(point) == -1)
		{
			Set_Status_Text(0, "");
			fCalPos = 0;
		}
		else
			Position_Calendar(point);
		Invalidate();
		CView::OnLButtonDown(nFlags, point);
		return;
	}
	// Determine if the user clicked a Day Box.
	for (row = 0; row < 6; row++)
	{
		for (col = 0; col < 7; col++)
		{
			if (m_rect[col][row].PtInRect(point))
			{
				if(!pDoc->GetSquare(col, row).IsEmpty())
				{
					if(fNote)
						Save_Note();
					// Invalidate the old Day Box so the border is removed.
					InvalidateRect(m_rect[nCurX][nCurY]);
					// Invalidate the new Day Box so the border is drawn.
					InvalidateRect(m_rect[col][row]);
					// Set Day integer to zero so it must be recalculated.
					nDay = 0;
					// Save our current box parameters.
					nCurX = col;
					nCurY = row;
					// Set the Edit flag so the Day Notes edit box gets updated.
					fEdit++;
				}
			}
		}
	}
	CView::OnLButtonDown(nFlags, point);
}

void CDaysEaseView::Position_Calendar(CPoint point)
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();

	// Set the tracker rectangle min size to half the current page dimensions.
	m_calendar.m_sizeMin.cx = theApp.page_rect.Width()/2;
	m_calendar.m_sizeMin.cy = theApp.page_rect.Height()/2;

	// Save the current calendar aspect ratio.
	m_calendar.Aspect = (double)m_calendar.m_rect.Width()/
		(double)m_calendar.m_rect.Height();
	// Save the current tracker rectangle size.
	m_calendar.hold_rect = m_calendar.m_rect;
	// If the Shift key is down, set the Shift flag.
	if(GetKeyState(VK_SHIFT) & 128)
		m_calendar.fShift++;
	// Start tracking the tracker rectangle.
	// Note that this function, CRectTracker::Track(), interacts with CRectTracker::AdjustRect().
	// See our CCalTracker class for the interaction.
	if(m_calendar.Track(this, point, FALSE))
	{
		// Update our new tracker rectangle dimensions.
		pDoc->ratio_right = m_calendar.m_rect.right/dCW;
		pDoc->ratio_left = m_calendar.m_rect.left/dCW;
		pDoc->ratio_top = (m_calendar.m_rect.top)/dCH;
		pDoc->ratio_bottom = (m_calendar.m_rect.bottom)/dCH;
		// Set the document modified flag.
		theApp.Set_Modified_Flag();
	}
	// Null the Shift flag.
	m_calendar.fShift = 0;
}

void CDaysEaseView::Position_Graphic(CPoint point)
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();

	// Save the current tracker rectangle size.
	m_graphic.hold_rect = m_graphic.m_rect;
	// If the Shift key is down, set the Shift flag.
	if(GetKeyState(VK_SHIFT) & 128)
		m_graphic.fShift++;
	// Start tracking the tracker rectangle.
	// Note that this function, CRectTracker::Track(), interacts with CRectTracker::AdjustRect().
	// See our CGraphTracker class for the interaction.
	if (m_graphic.Track(this, point, FALSE))
	{
		// Update our new tracker rectangle dimensions.
		pDoc->gratio_right = m_graphic.m_rect.right/dCW;
		pDoc->gratio_left = m_graphic.m_rect.left/dCW;
		pDoc->gratio_top = (m_graphic.m_rect.top)/dCH;
		pDoc->gratio_bottom = (m_graphic.m_rect.bottom)/dCH;
		theApp.Set_Modified_Flag();
	}
	// Null the Shift flag.
	m_graphic.fShift = 0;
}

void CDaysEaseView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	CMainFrame* pMain = theApp.Main_Pointer();
	int i, j;

	switch(nChar)
	{
	// Delete the Image.
	case VK_DELETE:
		if(fGrafPos)
		{
			// Call the CDib class to empty.
			pDoc->m_dib.Empty();
			pDoc->fGraphic = 0;
			fGrafPos = 0;
			Set_Status_Text(0, "");
			pDoc->csImageName = "no image loaded";
			pMain->m_ImageDlg.Enable_Controls(FALSE);
			theApp.Set_Modified_Flag();
			Invalidate();
			break;
		}
	// Cancel any positioning of Calendar or Image.
	case VK_ESCAPE:
		fCalPos = 0;
		fGrafPos = 0;
		Invalidate();
		break;
	// Remaining keys are for navigation in the View window.
	case VK_UP:
		if(Valid_Box(nCurX, nCurY -1))
			nCurY--;
		fEdit++;
		break;
	case VK_DOWN:
		if(Valid_Box(nCurX, nCurY +1))
			nCurY++;
		fEdit++;
		break;
	case VK_RIGHT:
		if(Valid_Box(nCurX +1, nCurY))
			nCurX++;
		else if(Valid_Box(0, nCurY +1))
		{
			nCurX = 0;
			nCurY++;
		}
		fEdit++;
		break;
	case VK_LEFT:
		if(Valid_Box(nCurX -1, nCurY))
			nCurX--;
		else if(Valid_Box(6, nCurY -1))
		{
			nCurX = 6;
			nCurY--;
		}
		fEdit++;
		break;
	case VK_TAB:
		if(GetKeyState(VK_SHIFT) & 128)
		{
			for(i = 0; i < 6; i++)
				if(Valid_Box(i, nCurY))
				{
					nCurX = i;
					fEdit++;
					i = 6;
				}
		}
		else for(i = 6; i > 0; i--)
				if(Valid_Box(i, nCurY))
				{
					nCurX = i;
					fEdit++;
					i = 0;
				}
			break;
	case VK_HOME:
		for(i = 0; i < 7; i++)
			if(Valid_Box(i, 0))
			{
				nCurX = i;
				nCurY = 0;
				fEdit++;
				i = 7;
			}
		break;
	case VK_END:
		for(j = 5; j > 2; j--)
			for(i = 6; i > -1; i--)
				if(Valid_Box(i, j))
				{
					nCurX = i;
					nCurY = j;
					fEdit++;
					i = -1;
					j = 2;
				}
		break;
	// Ignore Shift and Ctrl keys.
	case VK_SHIFT:
	case VK_CONTROL:
		break;
	default:
		// Make a sound if an improper key is pressed.
		MessageBeep(MB_ICONASTERISK);
	}	
	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CDaysEaseView::OnKillFocus(CWnd* pNewWnd) 
{
	CView::OnKillFocus(pNewWnd);
	
	
	// Cancel any positioning of Calendar or Image if focus leaves the View window.
	if(fCalPos || fGrafPos)
	{
		fCalPos = 0;
		// Don't cancel Image positioning if Image page is selected.
		if(!theApp.Main_Pointer()->m_ImageDlg.IsWindowVisible())
			fGrafPos = 0;
		Set_Status_Text(0, "");
		Invalidate();
	}
}

BOOL CDaysEaseView::Valid_Box(int x, int y)
{
	//Make sure the Day Box exists and has document data before moving to it.
	if(x < 0 || x > 6 || y < 0 || y > 5)
		return FALSE;
	if(theApp.Doc_Pointer()->GetSquare(x, y).IsEmpty())
		return FALSE;
	nDay = 0;
	// If we have a new note in the Day Notes edit box, save it.
	if(fNote)
		Save_Note();
	// Flag to redraw the old and new boxes.
	InvalidateRect(m_rect[nCurX][nCurY]);
	InvalidateRect(m_rect[x][y]);
	return TRUE;
}

BOOL CDaysEaseView::Cal_Hit_Test(CPoint point)
{
	if(m_calendar.HitTest(point) == CRectTracker::hitMiddle)
	{// Set the postion flag and draw the tracker rectangle border
		fCalPos++;
		fGrafPos = 0;
		fGontop = 0;
		CClientDC dc(this);
		dc.LPtoDP(m_calendar.m_rect);
		m_calendar.Draw(&dc);
		Invalidate();
		return TRUE;
	}
	return FALSE;
}

BOOL CDaysEaseView::Img_Hit_Test(CPoint point)
{
	if(m_graphic.HitTest(point) == CRectTracker::hitMiddle)
	{// Set the postion flag and draw the tracker rectangle border
		fGrafPos++;
		fCalPos = 0;
		CClientDC dc(this);
		dc.LPtoDP(m_graphic.m_rect);
		m_graphic.Draw(&dc);
		Invalidate();
		return TRUE;
	}
	return FALSE;
}

void CDaysEaseView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// If we have a new Day Note, save it.
	if(fNote)
		Save_Note();
	// Determine which Day Box the mouse pointer is on.
	for (row = 0; row < 6; row++)
	{
		for (col = 0; col < 7; col++)
		{
			if (m_rect[col][row].PtInRect(point))
			{
				CString string;

				// Get the document text for the Day Box.
				string = theApp.Doc_Pointer()->GetSquare(col, row);
				if(!string.IsEmpty())
				{
					// If Day number should go right, add a tab character in front.
					if(theApp.nDt_Swap == DT_RIGHT)
						string = (char)0x09 + string;
					// Add any events for the day.
					string += theApp.m_Eventgrid[col][row];
					// Put the text in our long "test" edit box and display it.
					theApp.Main_Pointer()->m_CalendarDlg.m_TextCheck.SetWindowText(string);
					theApp.Main_Pointer()->m_CalendarDlg.m_TextCheck.ShowWindow(SW_SHOW);
				}
			}
		}
	}
	
	CView::OnRButtonDown(nFlags, point);
}

void CDaysEaseView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	// Set Edit flag and invalidate the current Day Box so any new Day Note will show.
	fEdit++;
	InvalidateRect(m_rect[nCurX][nCurY]);
	// Hide our "Test" edit box.
	theApp.Main_Pointer()->m_CalendarDlg.m_TextCheck.ShowWindow(SW_HIDE);
	
	CView::OnRButtonUp(nFlags, point);
}

void CDaysEaseView::Save_Note()
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	CString New;

	// Get the text from the Day Notes edit box and save it in the document.
	theApp.Main_Pointer()->m_CalendarDlg.m_Data.GetWindowText(New);
	if(pDoc->GetSquare(nCurX, nCurY) != New)
	{
		pDoc->Add_Data(New, nCurX, nCurY);
		theApp.Set_Modified_Flag();
	}
	fNote = 0;
}

void CDaysEaseView::Create_Print_Dlg()
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
// If we already have our own print dialog, see if we need a new one.
	if(m_pPrintDialog != NULL)
	{// If the print dialog is for the document printer we don't need a new one, so bail out.
		if(m_pPrintDialog->GetDevMode()->dmDeviceName == pDoc->PrinterName)
			return;
	// Otherwise, get rid of the old one
		delete m_pPrintDialog;
		m_pPrintDialog = NULL;
	}
// Note that there is a way to programmatically switch printers,
// but in doesn't always work.  See the MS article "HOWTO: Obtaining the DEVMODE
// and DEVNAMES Structures."  Instead, I resorted to this, less than perfect, method of
// temporarily swapping the default printer, which seems to work every time.
	CString DefPrintString;
	BOOL trufal(0);
// Get the default printer string from Win.ini.
	GetProfileString("windows", "device", "",
		DefPrintString.GetBuffer(MY_BUF_SIZE), MY_BUF_SIZE);
	DefPrintString.ReleaseBuffer();
	CString DefPrintName(DefPrintString);
// Parse the string, we only need the printer name.
	if(DefPrintName.Find(",") != -1)
		DefPrintName = DefPrintName.Mid(0, DefPrintName.Find(","));
	if(DefPrintName != pDoc->PrinterName)
	{
		trufal = Validate_Printer();
		if(trufal)
		{// Temporarily swap out the default printer.
			WriteProfileString("windows", "device", pDoc->PrinterName);
		// Broadcast this message in case we fail to change the default printer info back.
			// Windows will conveniently fill in the driver and port info and make this
			// printer the default.  Send rather than Post, because we need to wait
			// until the new default is activated.
			::SendMessage(HWND_BROADCAST, WM_SETTINGCHANGE, NULL,
				(LPARAM)(LPCTSTR)"windows");
		}
		else
			MessageBox("The document was last saved with the following printer:\n\n" +
				pDoc->PrinterName +
				"\n\nwhich is not available.  The current printer will be used instead.",
				"DaysEase", MB_OK);
	}
// Create a new print dialog for our own use.
	m_pPrintDialog = new CPrintDialog(FALSE, PD_ALLPAGES, this);
	//UINT flags = PD_ALLPAGES;
	//flags |= PD_SELECTION;
	m_pPrintDialog->GetDefaults();
// GetDefaults() ORs the PD_RETURNDEFAULT flag, so remove it with XOR.
	m_pPrintDialog->m_pd.Flags ^= PD_RETURNDEFAULT;
	//m_pPrintDialog->m_pd.Flags = flags;
	m_pPrintDialog->m_pd.nCopies = 1;
	m_pPrintDialog->GetDevMode()->dmOrientation = theApp.Orientation;
	m_pPrintDialog->GetDevMode()->dmPaperSize = theApp.PaperSize;
	m_pPrintDialog->GetDevMode()->dmColor = theApp.PrintColor;
	ResetDC(m_pPrintDialog->m_pd.hDC, m_pPrintDialog->GetDevMode());
	Get_Printer_Stats(m_pPrintDialog->m_pd.hDC);
	if(trufal)
	{// Restore the default printer.
		WriteProfileString("windows", "device", DefPrintString);
	// Send, don't Post.  We must wait for Windows to update the default printer.
		::SendMessage(HWND_BROADCAST, WM_SETTINGCHANGE, NULL,
				(LPARAM)(LPCTSTR)"windows");
	}
}

void CDaysEaseView::OnFileNew() 
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	CMainFrame* pMain = theApp.Main_Pointer();

	// Display the common Save dialog if needed and
	// return without doing anything if the user selects Cancel.
	if(pDoc->IsModified())
		if(!pDoc->SaveModified())
			return;
	// Set the NewDoc flag.
	pDoc->fNewDoc++;
	// Since these are never less than zero, we'll know they need to be set later.
	nCurX = -1;
	nCurY = -1;
	// In case there is an image loaded, delete it.
	theApp.Main_Pointer()->m_ImageDlg.Delete_Image();
	pDoc->OnNewDocument();
	pDoc->SetModifiedFlag(FALSE);
	fEdit++;
	fEvents++;
	theApp.Main_Pointer()->m_CalendarDlg.Display_Month_Page();
	pMain->m_EventsDlg.Select_Event_File("Default");
	pMain->m_EventsDlg.Read_Events();
	Invalidate();
	// Remove document entries from DaysEase.ini.
	theApp.WriteProfileString("DaysEase", "Year Doc", "");
	theApp.WriteProfileString("DaysEase", "Month Doc", "");
}
