// EventsDlg.cpp : implementation file
//
/*
	DaysEase - PC Magazine file download utility
	Copyright  1998-1999 Ziff-Davis, Inc.  All rights reserved.
	First published in PC Magazine, US Edition
	December 1, 1998.
	Written by Richard A. Ellingson
*/
#include "stdafx.h"
#include "DaysEase.h"
#include "EventsDlg.h"
#include "MainFrm.h"
#include "DaysEaseDoc.h"
#include "DaysEaseView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
extern CDaysEaseApp theApp;
/////////////////////////////////////////////////////////////////////////////
// CEventsDlg dialog


CEventsDlg::CEventsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CEventsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CEventsDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CEventsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEventsDlg)
	DDX_Control(pDX, IDC_SORT, m_Sort);
	DDX_Control(pDX, IDC_REMOVEDATE, m_RemoveDate);
	DDX_Control(pDX, IDC_INSERTDATE, m_InsertDate);
	DDX_Control(pDX, IDC_EVENTINIBOX, m_EventIniBox);
	DDX_Control(pDX, IDC_EVENTSAVE, m_EventSave);
	DDX_Control(pDX, IDC_EVENTLIST, m_EventList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEventsDlg, CDialog)
	//{{AFX_MSG_MAP(CEventsDlg)
	ON_BN_CLICKED(IDC_INSERTDATE, OnInsertdate)
	ON_BN_CLICKED(IDC_REMOVEDATE, OnRemovedate)
	ON_CBN_SELCHANGE(IDC_EVENTINIBOX, OnSelchangeEventinibox)
	ON_CBN_EDITCHANGE(IDC_EVENTINIBOX, OnEditchangeEventinibox)
	ON_BN_CLICKED(IDC_EVENTSAVE, OnEventsave)
	//}}AFX_MSG_MAP
	ON_MESSAGE (WM_MY_UPDATE_EVENTS, OnUpdateEvents)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEventsDlg message handlers

void CEventsDlg::OnInsertdate() 
{
	// If the OK button was pressed or the Applied flag is set...
	if((IDOK == m_ChooseDateDlg.DoModal()) || fApplied)
	{
		CString text;

		m_EventSave.EnableWindow(TRUE);
		m_EventIniBox.GetWindowText(text);
		// Save the event file name in DaysEase.ini.
		// Note that this should be redundant, but it doesn't hurt to do it again.
		// FYI, Windows reads and writes to a cached copy of INI files and doesn't waste time
		// writing it to disk until the system is idle.
		Write_Last_File(text);
		// Save any changes.
		Write_Events();
		// Now read the changes and update the Event List.
		Read_Events();
		// Let the View know that new events exist.
		theApp.View_Pointer()->fEvents++;
		// Update the View.
		theApp.Update_View();
	}
	fApplied = 0;
}

BOOL CEventsDlg::OnUpdateEvents(UINT wParam, LPARAM lParam)
{
	CString text;

	// Next four lines are only conditional if we change some other functions.
	// The text should already be Null.
	theApp.Main_Pointer()->m_wndStatusBar.GetPaneText(1, text);
	theApp.Main_Pointer()->m_wndStatusBar.SetPaneText(1, "");
	if(text == " already listed")
		return FALSE;

	// Get the event date, add a tab character and the caption.
	text = theApp.EventDate[0] + (char)0x09 + m_ChooseDateDlg.m_EventCaption;
	// Bail out if the event is already listed.  This only happens during multiple selection
	// on the Holidays page.
	if(theApp.csEventList.Find(text) != -1)
		return FALSE;
	// Clear the list selection.
	m_EventList.SetSel(-1, FALSE);
	Adjust_Horz_Scroll(text);
	// Add the new event.
	m_EventList.AddString(text);
	// Sort the Event list.
	Sort_List();
	// Find the new event in the list, select it and scroll it into view.
	m_EventList.SetSel(m_EventList.FindStringExact(-1, text), TRUE);
	// Let the View know that new events exist.
	theApp.View_Pointer()->fEvents++;
	// Enable all buttons.
	Enable_Buttons(TRUE);
	// Set the Save button text to Delete, because we know the event file has been saved.
	m_EventSave.SetWindowText("Delete");
	// Set the Applied flag so we'll know that the View should be updated.
	fApplied++;
	// This function always returns FALSE just to let us know that it's been called.
	return FALSE;
}

BOOL CEventsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	// Intitialize this flag.
	fApplied = 0;
	// Set the Event List tab stop so the captions appear in a column.
	m_EventList.SetTabStops(MY_TAB_STOP);
	// Limit the Event file name combo box so we don't crash
	// when someone copies their resum into it.
	m_EventIniBox.LimitText(64);
	// Intitialize the Event file name combo box.
	Fill_EventIni_Box();
	// Update the Event list.
	Read_Events();
	return TRUE;
}

void CEventsDlg::Write_Events()
{
	int count, i;
	CString value, text;
	char item[20];

	// Get the index of the current event file name.
	count = m_EventIniBox.GetCurSel();
	// We could get an error if the user has typed a new name.
	if(count == CB_ERR)
	{
		// Get the combo edit box text.
		m_EventIniBox.GetWindowText(text);
		// If there is text, see if we have a listed event file by that name.
		if(!text.IsEmpty())
		{
			// If not, add the new name to the list and select it.
			if(m_EventIniBox.FindStringExact(-1, text) == CB_ERR)
				m_EventIniBox.AddString(text);
			Select_Event_File(text);
		}
	}
	else  // If we have an index, get the name from the list.
		m_EventIniBox.GetLBText(count, text);
	// If we now have an event file name, rewrite it or create a new file.
	if(!text.IsEmpty())
	{
		// Give it and extension.
		text += ".evn";
		// See if it was specified on the commandline at launch time.
		// Note that any event file specified on the commandline has priority over one with
		// the same name in the current directory.
		if(theApp.m_csCmdLineIni.Find("\\" + text) != -1)
			text = theApp.m_csCmdLineIni;
		else  // If not on commandline then it's in the current directory.
			text = theApp.m_csCurrentDir + text;
		// Get the total count of events in the Event list.
		count = m_EventList.GetCount();
		for(i = 0; i < count; i++)
		{
			// Convert integer i to a character.
			itoa(i, item, 10);
			// Get the text for each event from the Event List.
			m_EventList.GetText(i, value);
			Adjust_Horz_Scroll(value);
			// Replace the tab character with a semicolon.
			// Note that tab characters are not allowed in INI files.
			value = value.Mid(0, value.Find((char)0x09)) + ";" + 
				value.Mid(value.Find((char)0x09) +1);
			// Now write the event.
			WritePrivateProfileString("Events", item, value, text);
		}
	}
}

void CEventsDlg::Fill_EventIni_Box()
{
	HANDLE find_handle = {NULL};
	WIN32_FIND_DATA find;
	CString current_file;

	// Find the first file in the current directory with the .evn extension.
	find_handle = FindFirstFile(theApp.m_csCurrentDir + "*.evn", &find);
	// Keep looking until all have been found.
	while(find_handle != INVALID_HANDLE_VALUE)
	{
		// Get the file name.
		current_file = find.cFileName;
		// Remove the extension.
		current_file = current_file.Mid(0, current_file.GetLength() - 4);
		// Make sure the file actually has a name, not just an extension.
		// Also, be sure it's not a directory with the .evn extension.
		if(!current_file.IsEmpty() &&
			(find.dwFileAttributes != (find.dwFileAttributes | FILE_ATTRIBUTE_DIRECTORY)))
		{
			// Add the name to the Event file name combo box.
			m_EventIniBox.AddString(current_file);
			// We have at least one, so enable the buttons.
			Enable_Buttons(TRUE);
		}
		// If there are no more files, then close the handle and break the while loop.
		if(!FindNextFile(find_handle, &find))
		{
			FindClose(find_handle);
			break;
		}
	}
	// Get the commandline eventfile name and remove the path.
	current_file = theApp.m_csCmdLineIni.Mid(theApp.m_csCmdLineIni.ReverseFind('\\') +1);
	// Remove the .evn extension.
	current_file = current_file.Mid(0, current_file.GetLength() - 4);
	// Make sure we still have a file name.
	if(!current_file.IsEmpty())
	{
		// If it's not already listed, add it to the Event file name combo box.
		// Note that there may be another event file in the current directory with the same name
		// which is already listed.  Since we always check for the commandline event file first,
		// the one in the current directory will be ignored.
		if(m_EventIniBox.FindStringExact(-1, current_file) == CB_ERR)
			m_EventIniBox.AddString(current_file);
	}
	// Select the default event file, if any.
	Select_Event_File("Default");

}

void CEventsDlg::Read_Events()
{
	CString value, text;
	char item[20];
	int i(0);
	int index;
	int count;
	TCHAR szProfile[MAX_PATH];

	// Empty the Event List and the CString  that contains the entire list.
	m_EventList.ResetContent();
	theApp.csEventList.Empty();
	m_EventList.SetHorizontalExtent(0);
	// Empty the array that contains event captions for the currently selected month.
	theApp.Empty_Event_Array();
	// Get the index of the currently selected event file, or bail out if none.
	if((index = m_EventIniBox.GetCurSel()) == CB_ERR)
		return;
	// Get the name of the file.
	m_EventIniBox.GetLBText(index, text);
	// Bail out if some error caused a blank entry.
	if(text.IsEmpty())
		return;
	// Add the extension to the event file name.
	text += ".evn";
	// Check to see if it's the one on the commandline.
	if(theApp.m_csCmdLineIni.Find("\\" + text) != -1)
		text = theApp.m_csCmdLineIni;
	else  // If not it must exist in the current directory, so add the path.
		text = theApp.m_csCurrentDir + text;
	// Get the events, at least the first 500, anyway.
	// We could allow any number of events in a file, but 500 seemed a reasonable limit.
	while(i < 500)
	{
		// Convert the integer i to a character string.
		itoa(i, item, 10);
		// Get the event.
		GetPrivateProfileString("Events", item, "END",
			szProfile, sizeof(szProfile), text);
		// Copy it to a CString so it's easier to work with.
		value = szProfile;
		// If it equals our default return string, then we reached the end, so break the while loop.
		if(value == "END")
			break;
		// Correct if numerals are single digit.
		// Since the event files are actually INI files, they are easy to edit,
		// and that would be a common error.
		if(value.GetAt(3) != '-')
			if(CString(value.GetAt(5)).FindOneOf("0123456789") == -1)
				if(value.Left(6) != "EASTER")
					value = value.Left(4) + "0" + value.Mid(4);
		// Substitute a tab character in place of the semicolon.
		// Note that INI files can not contain tab characters.
		value = value.Mid(0, value.Find(";")) + (char)0x09 + 
			value.Mid(value.Find(";") +1);
		Adjust_Horz_Scroll(value);
		// Add it to our Event List.
		m_EventList.AddString(value);
		// Increment our counter.
		i++;
	}
	// Perform the appropriately named function.
	Sort_List();
	// Empty the CString that we store the formatted list in.
	theApp.csEventList.Empty();
	// Get the total count of items in the Event List.
	count = m_EventList.GetCount();
	// Add each event to the CString, separated by end of line characters.
	for(i = 0; i < count; i++)
	{
		m_EventList.GetText(i, text);
		theApp.csEventList += text + (char)0x0d + (char)0x0a;
	}
	// If we have at least one event in the list...
	if(count)
	{
		// Enable the Remove Date button.
		m_RemoveDate.EnableWindow(TRUE);
		// Select the first one.
		m_EventList.SetSel(0, TRUE);
	}
	else // otherwise disable the Remove Date button.
		m_RemoveDate.EnableWindow(FALSE);
}

void CEventsDlg::OnRemovedate() 
{
	int i(0);
	int count;
	int index;
	const int MAX = 100;
	int rgIndex[MAX];
	CString text;

	// Get an array of selected events.  If none, then an error occurred, so bail out.
	if(m_EventList.GetSelItems(MAX, rgIndex) == 0)
		return;
	m_EventList.SetHorizontalExtent(0);
	// Save the position just before the selected event.
	index = rgIndex[0] -1;
	// If it's negative...
	if(index == -1)
		index = 0;
	// Delete the first event and repeat until none.
	while(m_EventList.GetSelItems(MAX, rgIndex))
		m_EventList.DeleteString(rgIndex[0]);
	// If the Event list is empty set the button states.
	if((count = m_EventList.GetCount()) == 0)
	{
		m_EventSave.SetWindowText("Delete");
		m_EventSave.EnableWindow(TRUE);
		m_InsertDate.EnableWindow(TRUE);
		m_RemoveDate.EnableWindow(FALSE);
	}
	else // Otherwise select the event previous to the first deleted.
		m_EventList.SetSel(index, TRUE);
	// Get the current event file name.
	m_EventIniBox.GetWindowText(text);
	// Add the extension.
	text += ".evn";
	// Check to see if it's the one on the commandline.
	if(theApp.m_csCmdLineIni.Find("\\" + text) != -1)
		text = theApp.m_csCmdLineIni;
	else // Otherwise it belongs to the current directory, so add the path.
		text = theApp.m_csCurrentDir + text;
	// Empty the events from the event file.
	// Note, passing NULL as second parameter removes all entries in the section.
	WritePrivateProfileSection("Events", NULL, text);
	// Rewrite the events to the file.
	Write_Events();
	// Empty the month event array.
	theApp.Empty_Event_Array();
	// Empty the CString that we store the formatted list in.
	theApp.csEventList.Empty();
	// Add each event to the CString, separated by end of line characters.
	for(i = 0; i < count; i++)
	{
		m_EventList.GetText(i, text);
		theApp.csEventList += text + (char)0x0d + (char)0x0a;
	}
	// Update the View.
	theApp.Update_View();
}

void CEventsDlg::OnSelchangeEventinibox() 
{
	int index;
	CString text;

	// Get the currently selected event index and bail out if there is none.
	index = m_EventIniBox.GetCurSel();
	if(index == CB_ERR)
		return;
	// Get the Event file name.
	m_EventIniBox.GetLBText(index, text);
	// If an error occurred, disable the buttons and bailout.
	if(text.IsEmpty())
	{
		Enable_Buttons(FALSE);
		return;
	}
	else // Otherwise enable the buttons.
		Enable_Buttons(TRUE);
	// Save the event file name in DaysEase.ini.
	Write_Last_File(text);
	// Read the file.
	Read_Events();
	// Let the View know that we have new events and update it.
	theApp.View_Pointer()->fEvents++;
	theApp.Update_View();
}

void CEventsDlg::OnEditchangeEventinibox() 
{
	CString text;
	int index, i(0);

	// Get the text in the Event file name combo box.
	m_EventIniBox.GetWindowText(text);
	// If it's not empty, we need to get rid of any characters that are illegal for file names.
	if(!text.IsEmpty())
	{
		// If the user types a space, or pastes several spaces, at the beginning of the Event File name,
		// the next while loop will not remove it, because it's an acceptable character.
		// If we used the CString::TrimLeft() function we have no way of knowing if we modified the text.
		// We don't want to just replace the text each time if there's nothing wrong,
		// because the text will flash each time.  Instead, we'll replace the preceding spaces with
		// question marks and then let the next while loop fix it.  Trailing spaces are no problem,
		// we remove them before saving the file.
		while(text.GetAt(i) == ' ')
		{
			text.SetAt(i, '?');
			i++;
			// Text could be nothing but spaces, if so, break.
			if(i >= text.GetLength())
				break;
		}
		// Null the counter.
		i = 0;
		// While we're at it, include the Ctrl + Backspace character, (0x7f), as unwanted.
		while((index = text.FindOneOf("\\/:*?\"<>|" + CString(0x7f))) != -1)
		{
			text = text.Mid(0, index) + text.Mid(index +1);
			m_EventIniBox.SetWindowText(text);
			m_EventIniBox.SetEditSel(index, index);
			i++;
		}
		// Null the counter.
		i = 0;
	}
	// Text could be empty after we modified it.
	if(text.IsEmpty())
	{
		// Use the counter to flag that the text is empty.
		i = 1;
		m_EventIniBox.SetCurSel(-1);
		// Empty the Event List.
		m_EventList.ResetContent();
		m_EventList.SetHorizontalExtent(0);
		// Empty the month event array.
		theApp.Empty_Event_Array();
		// Empty the Event string.
		theApp.csEventList.Empty();
		// Disable the buttons.
		Enable_Buttons(FALSE);
		// Update the View.
		theApp.Update_View();
	}
	else  // If not, enable the Insert Date button.
		m_InsertDate.EnableWindow(TRUE);
	// If we have events in the Event List, enable the buttons.
	if(m_EventList.GetCount())
		Enable_Buttons(TRUE);
	else  // Otherwise, disable Save and Remove Date buttons.
	{
		m_EventSave.EnableWindow(FALSE);
		m_RemoveDate.EnableWindow(FALSE);
	}
	// If we flagged empty, we're done.
	if(i == 1)
	{
		// Clear the Last Event File from DaysEase.ini.
		Write_Last_File("");
		return;
	}
	// See if our text is already listed in the Event File name combo box.
	index = m_EventIniBox.FindStringExact(-1, text);
	// If it's there, enable buttons accordingly.
	if(index != CB_ERR)
	{
		m_EventSave.EnableWindow(TRUE);
		m_InsertDate.EnableWindow(FALSE);
		m_RemoveDate.EnableWindow(FALSE);
		m_EventSave.SetWindowText("Load");
	}
	// Trim off any trailing spaces and check again.
	text.TrimRight();
	index = m_EventIniBox.FindStringExact(-1, text);
	// If its not there, but we have events, then we have a potentially new event file name,
	// so enable buttons accordingly.
	if(index == CB_ERR && m_EventList.GetCount())
	{
		Enable_Buttons(TRUE);
		m_EventSave.SetWindowText("Save");
		// User must Save before Removing Dates, so disable button.
		m_RemoveDate.EnableWindow(FALSE);
	}
}
// Save, Delete, Load button handler
void CEventsDlg::OnEventsave() 
{
	CString state, text;
	int index;

	// Get the Save, Delete, Load button text.
	m_EventSave.GetWindowText(state);
	// Get the current Event File name, which may be new.
	m_EventIniBox.GetWindowText(text);
	// Trim off leading and trailing spaces, in case the user has typed them.
	text.TrimLeft();
	text.TrimRight();
	// See if it's already in the Event File name combo box.
	index = m_EventIniBox.FindStringExact(-1, text);
	// If the button state is...
	if(state == "Load")
	{
		// If the Event file name is already in the combo box...
		if(index != CB_ERR)
		{
			// Select it.
			Select_Event_File(text);
			// Enable all buttons.
			Enable_Buttons(TRUE);
			// Read the file.
			Read_Events();
			// Set the button text.
			m_EventSave.SetWindowText("Delete");
		}
		return;
	}
	// If the button state is...
	if(state == "Save")
	{
		// If the Event file name is not in the combo box, add it.
		if(index == CB_ERR)
			m_EventIniBox.AddString(text);
		// Select it.
		Select_Event_File(text);
		// Set the button text.
		m_EventSave.SetWindowText("Delete");
		// Write the event to the new event file.
		Write_Events();
	}
	// If the button state is...
	if(state == "Delete")
	{
		// Add the extension to the Event File name.
		text += ".evn";
		// Ask user to confirm the file delete and bail out if NO.
		if(IDNO == MessageBox("Please confirm that you want to delete the file:\n\n     " + text,
			"DaysEase", MB_YESNO|MB_TOPMOST))
			return;
		// Make sure the name is in the Event File name combo box and then delete it.
		if(index != CB_ERR)
				m_EventIniBox.DeleteString(index);
		// Check to see if it's the event file given on the commandline at launch time.
		if(theApp.m_csCmdLineIni.Find("\\" + text) != -1)
		{
			text = theApp.m_csCmdLineIni;
			// Empty the commandline event file string, since we are going to delete the file.
			theApp.m_csCmdLineIni.Empty();
		}
		else  // If not, then it's in the current directory, so add the path.
			text = theApp.m_csCurrentDir + text;
		// Delete the file.
		DeleteFile(text);
		// If this is the Last Event File in DaysEase.ini, clear the entry.
		if(text == Get_Last_File())
			Write_Last_File("");
		Select_Event_File("Default");
		Read_Events();
		theApp.Update_View();
	}
	OnSelchangeEventinibox();
}

void CEventsDlg::OnCancel()
{
	// Do nothing, so that the window doesn't disappear when the user presses Esc.
}

void CEventsDlg::Enable_Buttons(BOOL trufal)
{
	m_EventSave.EnableWindow(trufal);
	m_InsertDate.EnableWindow(trufal);
	m_RemoveDate.EnableWindow(trufal);
}

void CEventsDlg::Write_Last_File(CString text)
{
	if(!text.IsEmpty())
	{
		// Add the extension.
		text += ".evn";
		// If it's the one on the commandline make text the commandline in DaysEase.ini.
		if(theApp.m_csCmdLineIni.Find("\\" + text) != -1)
			text = theApp.m_csCmdLineIni;
		else  // Otherwise, add the current directory path.
			text = theApp.m_csCurrentDir + text;
	}
	// Save the event file name in DaysEase.ini.
	theApp.WriteProfileString("DaysEase", "Last Event File", text);
}

void CEventsDlg::OnOK()
{
	// Do nothing, so that the window doesn't disappear when the user presses Enter.
}

void CEventsDlg::Sort_List()
{
	CString month[13] = {"Jan", "Feb", "Mar", "Apr", "EASTER", "May", "Jun",
		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
	CString text;
	int i, j, index;

	// Note; m_Sort is an invisible CListBox used, temporarily, to sort alphabetically
	for(i = 0; i < 13; i++)
	{
		// Empty the temorary ListBox.
		m_Sort.ResetContent();
		while((index = m_EventList.FindString(-1, month[i])) != LB_ERR)
		{
			m_EventList.GetText(index, text);
			m_EventList.DeleteString(index);
			m_Sort.AddString(text);
		}
		if((index = m_Sort.GetCount()) != LB_ERR)
		for(j = 0; j < index; j++)
		{
			m_Sort.GetText(j, text);
			Adjust_Horz_Scroll(text);
			m_EventList.AddString(text);
		}
	}
}

BOOL CEventsDlg::Select_Event_File(CString filename)
{
	int index;

	if(filename.IsEmpty())
		return FALSE;
	if(filename == "Default")
	{
		// See if we have a last file in DaysEase.ini.
		filename = Get_Last_File();
		// If not, check the commandline.
		if(filename.IsEmpty())
			filename = theApp.m_csCmdLineIni;
		// If still not, setup and bail.
		if(filename.IsEmpty())
		{
			m_EventIniBox.Clear();
			m_EventIniBox.SetWindowText("");
			Enable_Buttons(FALSE);
			return FALSE;
		}
		// Extract the name from the path.
		filename = filename.Mid(filename.ReverseFind('\\') +1);
		filename = filename.Mid(0, filename.GetLength() -4);
	}
	// Get the index of the event file name from the Event File name combo box.
	if((index = m_EventIniBox.FindStringExact(-1, filename)) != CB_ERR)
	{
		// Write the file name to DaysEase.ini.
		Write_Last_File(filename);
		// Select the event file in the combo box.
		m_EventIniBox.SetCurSel(index);
		return TRUE;
	}
	return FALSE;
}

CString CEventsDlg::Get_Last_File()
{
	return theApp.GetProfileString("DaysEase", "Last Event File", "");
}

// This function added to version 1.1 to enable horizontal scroll for the Events List.
void CEventsDlg::Adjust_Horz_Scroll(CString text)
{
	CSize       size;
	CDC*        pDC;
    CFont*       pOldFont;
	int tab = MY_TAB_STOP;

	pDC = m_EventList.GetDC();
	pOldFont = pDC->SelectObject(m_EventList.GetFont());
	size = pDC->GetTabbedTextExtent(text + "o", 1, &tab);
	if(m_EventList.GetHorizontalExtent() < size.cx)
		m_EventList.SetHorizontalExtent(size.cx);
	pDC->SelectObject(pOldFont);
	ReleaseDC(pDC);
}
