// MainFrm.cpp : implementation of the CMainFrame class
//
/*
	DaysEase - PC Magazine file download utility
	Copyright  1998-1999 Ziff-Davis, Inc.  All rights reserved.
	First published in PC Magazine, US Edition
	December 1, 1998.
	Written by Richard A. Ellingson
*/
#include "stdafx.h"
#include "DaysEase.h"

#include "MainFrm.h"
#include "DaysEaseDoc.h"
#include "DaysEaseView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#define MY_MIN_WIDTH 640
#define MY_MIN_HEIGHT 456
extern CDaysEaseApp theApp;
/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_GETMINMAXINFO()
	ON_WM_SIZE()
	ON_NOTIFY(TCN_SELCHANGE, IDC_MDBTAB, OnSelchangeMdbtab)
	ON_WM_SETCURSOR()
	ON_WM_SIZING()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
	// Global help commands
	ON_COMMAND(ID_HELP_FINDER, CFrameWnd::OnHelpFinder)
	ON_COMMAND(ID_HELP, CFrameWnd::OnHelp)
	ON_COMMAND(ID_CONTEXT_HELP, CFrameWnd::OnContextHelp)
	ON_COMMAND(ID_DEFAULT_HELP, CFrameWnd::OnHelpFinder)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_SEPARATOR,
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// Save the Main window pointer.
	theApp.Set_Main_Pointer(this);
	
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	/* AppWizard generated code.  We don't need the toolbar.
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}
	*/
	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}
	// Create the DialogBar where our tabbed pages will reside.
	if(!m_MainDlgBar.Create(this, IDD_DIALOGBAR, CBRS_RIGHT|CBRS_FLYBY, IDD_DIALOGBAR))
	{
		TRACE0("Failed to create dialog bar\n");
		return -1;      // fail to create
	}
	// Create the tab control.
	TC_ITEM TabCtrlItem;
	TabCtrlItem.mask = TCIF_TEXT;
	LPSTR item[4] = {"Calendar", "Style", "Image", "Events"};
	CTabCtrl* pTabCtrl = ((CTabCtrl*)m_MainDlgBar.GetDlgItem(IDC_MDBTAB));
	int i;
	for(i=0; i<4; i++)
	{
		TabCtrlItem.pszText = item[i];
		pTabCtrl->InsertItem( i, &TabCtrlItem );
	}
	// Create the tabbed page dialogs on the tab control.
	m_CalendarDlg.Create(IDD_CALENDARDLG, pTabCtrl);
	m_StyleDlg.Create(IDD_STYLEDLG, pTabCtrl);
	m_ImageDlg.Create(IDD_IMAGEDLG, pTabCtrl);
	m_EventsDlg.Create(IDD_EVENTSDLG, pTabCtrl);
	// Show the Calendar page as default.
	m_CalendarDlg.ShowWindow(SW_SHOW);
	// Initialize the status bar.
	m_wndStatusBar.SetPaneText(0, "Ready");
	// TODO: Remove this if you don't want tool tips or a resizeable toolbar
	//m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
	//	CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	//m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	//EnableDocking(CBRS_ALIGN_ANY);
	//DockControlBar(&m_wndToolBar);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	RECT rect;
	
	// Get the last window dimensions, if available in DaysEase.ini.
	cs.x = theApp.GetProfileInt("DaysEase", "Window PosX", 0);
	cs.y = theApp.GetProfileInt("DaysEase", "Window PosY", 0);
	cs.cx = theApp.GetProfileInt("DaysEase", "Window SizeX", MY_MIN_WIDTH);
	cs.cy = theApp.GetProfileInt("DaysEase", "Window SizeY", MY_MIN_HEIGHT);

	// Get the Desktop size and set the DaysEase window size and position.
	if(SystemParametersInfo(SPI_GETWORKAREA, 0, &rect, 0))
	{
		if(cs.x < 0)
			cs.x = 0;
		if(cs.x > rect.right - MY_MIN_WIDTH)
			cs.x = rect.right - MY_MIN_WIDTH;
		if(cs.y < 0)
			cs.y = 0;
		if(cs.y > rect.bottom - MY_MIN_HEIGHT)
			cs.y = rect.bottom - MY_MIN_HEIGHT;
		if((cs.x + cs.cx) > rect.right)
		{
			cs.cx = MY_MIN_WIDTH;
			cs.x = rect.right - MY_MIN_WIDTH;
		}
		if((cs.y + cs.cy) > rect.bottom)
		{
			cs.cy = MY_MIN_HEIGHT;
			cs.y = rect.bottom - MY_MIN_HEIGHT;
		}
	}
	return CFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
	// Make sure user can't make window too small
	lpMMI->ptMinTrackSize.x = MY_MIN_WIDTH;
	lpMMI->ptMinTrackSize.y = MY_MIN_HEIGHT;
	
	CFrameWnd::OnGetMinMaxInfo(lpMMI);
}

void CMainFrame::OnSize(UINT nType, int cx, int cy) 
{
	CFrameWnd::OnSize(nType, cx, cy);
	
	// Make sure we have a main window and then size the tabbed pages.
	if(IsWindowVisible())
		Position_Page();
}

void CMainFrame::OnSelchangeMdbtab(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TabCtrl_GetCurSel() macro returns zero based integer of selected tab item.
	Change_Tab_Page(TabCtrl_GetCurSel(pNMHDR->hwndFrom));
	*pResult = 0;
}

CDialog* CMainFrame::Page_Visible()
{
	// Return a pointer to the visible page.  All other pages are hidden.
	if(m_CalendarDlg.IsWindowVisible())
		return &m_CalendarDlg;
	if(m_StyleDlg.IsWindowVisible())
		return &m_StyleDlg;
	if(m_ImageDlg.IsWindowVisible())
		return &m_ImageDlg;
	if(m_EventsDlg.IsWindowVisible())
		return &m_EventsDlg;
	return NULL;
}

void CMainFrame::Position_Page()
{
	// Bail out if our DialogBar isn't ready.
	if(!m_MainDlgBar.IsWindowVisible())
		return;
	CTabCtrl* pTabCtrl = (CTabCtrl*)m_MainDlgBar.GetDlgItem(IDC_MDBTAB);
	CRect tabrect;

	// Get the main window client rectangle.
	// Note; top and left are always zero, while right and bottom
	// contain the width and height of the window.
	GetClientRect(tabrect);
	// Move the Tab control to this rectangle.
	// Since the tab control is a child window of the DialogBar,
	// the top and left, both zero, will correspond to the top and left of the DialogBar.
	// We don't care that the right and bottom sides extend past the main window,
	// they are automatically truncated.
	pTabCtrl->MoveWindow(tabrect, TRUE);
	// Get the DialogBar window rectangle.
	m_MainDlgBar.GetWindowRect(tabrect);
	// Calculate a size for each tab in the control.
	CSize size((tabrect.Width() +4)/4,20);
	// Set the tab size for all four tabs.
	pTabCtrl->SetItemSize(size);
	// Get the tab control's client rectangle, including tabs and borders.
	pTabCtrl->GetClientRect(tabrect);
	// Pass the tab control's client rectangle to AdjustRect() with the first parameter FALSE.
	// It returns the rectangle of the tab control's display area in tabrect.
	pTabCtrl->AdjustRect(FALSE, tabrect);
	// Now make the visible page the same as the tab control's display area.
	Page_Visible()->MoveWindow(tabrect, TRUE);
}

BOOL CMainFrame::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	CString text(theApp.Doc_Pointer()->PrinterName);

	// If the mouse pointer is over one of the Font buttons,
	// display the font name in the status bar.
	// First, make sure the Style page is visible.
	if(m_StyleDlg.IsWindowVisible())
	{
		if(pWnd == &m_StyleDlg.m_btnWeekDay)
			text = theApp.csWkdayFont;
		if(pWnd == &m_StyleDlg.m_btnData)
			text = theApp.csNotesFont;
		if(pWnd == &m_StyleDlg.m_btnTitle)
			text = theApp.csTitleFont;
		if(pWnd == &m_StyleDlg.m_btnDate)
			text = theApp.csDateFont;
	}
	// If text is unchanged, it contains the current printer name.
	m_wndStatusBar.SetPaneText(1, text);

	return CFrameWnd::OnSetCursor(pWnd, nHitTest, message);
}

void CMainFrame::OnSizing(UINT fwSide, LPRECT pRect) 
{
	CFrameWnd::OnSizing(fwSide, pRect);
	
	theApp.fSizing++;
}

void CMainFrame::OnClose() 
{
	CString state;

	m_EventsDlg.m_EventSave.GetWindowText(state);
	if(state == "Save")
	{
		CString text;
		m_EventsDlg.m_EventIniBox.GetWindowText(text);
		if(!text.IsEmpty())
			if(IDYES == MessageBox("Save Event file: " + text + ".evn", "DaysEase", MB_YESNO))
				m_EventsDlg.OnEventsave();
	}
	RECT rect;
	UINT nShow(SW_SHOW);
	
	// Save the maximized state in DaysEase.ini if the window is maximized.
	if(IsZoomed())
		nShow = SW_SHOWMAXIMIZED;
	else  // If not minimized, save the size and position.
	if(!IsIconic())
	{
		GetWindowRect(&rect);
		theApp.WriteProfileInt("DaysEase", "Window PosX", rect.left);
		theApp.WriteProfileInt("DaysEase", "Window PosY", rect.top);
		theApp.WriteProfileInt("DaysEase", "Window SizeX", rect.right - rect.left);
		theApp.WriteProfileInt("DaysEase", "Window SizeY", rect.bottom - rect.top);
	}
	theApp.WriteProfileInt("DaysEase", "Show Window", nShow);
	CFrameWnd::OnClose();
}

LRESULT CMainFrame::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	CDaysEaseView* pView = theApp.View_Pointer();


	// First make sure our pointers are valid, they are not valid when the program first starts.
	if(pView && m_MainDlgBar)
	{
		// If the image is selected and the user types the Delete key, delete the image.
		if((pView->fGrafPos) && (GetKeyState(VK_DELETE) & 128))
		m_ImageDlg.Delete_Image();
		// If user types our defined shortcut key, Ctrl + Alt, take appropriate action.
		if(GetKeyState(VK_MENU) & 128)
		{
			int page(4);

			// VK_V = 43h, set focus to the View.
			if(GetKeyState(0x56) & 128)
				::SetFocus(pView->m_hWnd);
			// VK_C = 43h, set Calendar page active.
			if(GetKeyState(0x43) & 128)
				page = 0;
			// VK_S = 53h, set Style page active.
			if(GetKeyState(0x53) & 128)
				page = 1;
			// VK_I = 49h, set Image page active.
			if(GetKeyState(0x49) & 128)
				page = 2;
			// VK_E = 45h, set Events page active.
			if(GetKeyState(0x45) & 128)
				page = 3;
			if(page < 4)
			{
				// Set the tab control.
				TabCtrl_SetCurSel(m_MainDlgBar.GetDlgItem(IDC_MDBTAB)->GetSafeHwnd(), page);
				// Change the page.
				Change_Tab_Page(page);
			}
		}
	}
	return CFrameWnd::WindowProc(message, wParam, lParam);
}

void CMainFrame::Change_Tab_Page(int page)
{
	CDaysEaseDoc* pDoc = theApp.Doc_Pointer();
	CDaysEaseView* pView = theApp.View_Pointer();

	Page_Visible()->ShowWindow(SW_HIDE);
	if(pView->fGontop || pView->fGrafPos)
	{
		pView->fGontop = 0;
		pView->fGrafPos = 0;
		theApp.Update_View();
	}
	switch(page)
	{
	case 0:
		m_CalendarDlg.ShowWindow(SW_SHOW);
		m_CalendarDlg.SetFocus();
		break;
	case 1:
		m_StyleDlg.ShowWindow(SW_SHOW);
		m_StyleDlg.SetFocus();
		break;
	case 2:
		if(pDoc->fGraphic && !pView->fGrafPos)
		{
			pView->fCalPos = 0;
			pView->fGontop++;
			pView->fGrafPos++;
			theApp.Update_View();
		}
		m_ImageDlg.ShowWindow(SW_SHOW);
		m_ImageDlg.SetFocus();
		break;
	case 3:
		m_EventsDlg.ShowWindow(SW_SHOW);
		m_EventsDlg.SetFocus();
	}
	Position_Page();
}
